from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.docker.container.up_command import up_command

runner = CliRunner()


class TestUpCommand:
    """Tests pour la commande docker compose up"""

    def test_up_default(self):
        """command n7 dkc up -> docker compose up -d"""
        with patch("n7.commands.docker.container.up_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.up_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(up_command, [])

                assert result.exit_code == 0
                mock_run.assert_called_once()
                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "up",
                    "-d",
                )

    def test_up_without_detach(self):
        """command n7 dkc up --no-detach"""
        with patch("n7.commands.docker.container.up_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.up_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(up_command, ["--no-detach"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "-d" not in cmd
                assert cmd[-1] == "up"

    def test_up_custom_files(self):
        """n7 dkc up avec fichiers custom via n7.yaml"""
        with patch("n7.commands.docker.container.up_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.up_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env.prod",
                    "compose_file": "docker-compose.prod.yml",
                    "default_service": None,
                }

                result = runner.invoke(up_command, [])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert ".env.prod" in cmd
                assert "docker-compose.prod.yml" in cmd



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.docker.format.docker_ruff_command import docker_ruff_command

runner = CliRunner()


class TestDockerRuffCommand:
    """Tests pour la commande docker ruff"""

    def test_command_without_options(self):
        """n7 dkc pyrf -> ruff check simple dans container"""
        with patch("n7.commands.docker.format.docker_ruff_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_ruff_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_ruff_command, [])

                assert result.exit_code == 0
                mock_run.assert_called_once()
                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "api",
                    "ruff",
                    "check",
                )

    def test_command_with_path(self):
        """n7 dkc pyrf src/"""
        with patch("n7.commands.docker.format.docker_ruff_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_ruff_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_ruff_command, ["src/"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert cmd[-1] == "src/"

    def test_command_with_fix(self):
        """n7 dkc pyrf --fix"""
        with patch("n7.commands.docker.format.docker_ruff_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_ruff_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_ruff_command, ["--fix"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--fix" in cmd

    def test_command_with_diff(self):
        """n7 dkc pyrf --diff"""
        with patch("n7.commands.docker.format.docker_ruff_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_ruff_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_ruff_command, ["--diff"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--diff" in cmd

    def test_command_with_select(self):
        """n7 dkc pyrf --select E,F"""
        with patch("n7.commands.docker.format.docker_ruff_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_ruff_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_ruff_command, ["--select", "E,F"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--select" in cmd
                assert "E,F" in cmd

    def test_command_with_ignore(self):
        """n7 dkc pyrf --ignore E501"""
        with patch("n7.commands.docker.format.docker_ruff_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_ruff_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_ruff_command, ["--ignore", "E501"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--ignore" in cmd
                assert "E501" in cmd

    def test_command_with_verbose(self):
        """n7 dkc pyrf -v"""
        with patch("n7.commands.docker.format.docker_ruff_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_ruff_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_ruff_command, ["-v"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--verbose" in cmd

    def test_command_with_custom_service(self):
        """n7 dkc pyrf --service backend"""
        with patch("n7.commands.docker.format.docker_ruff_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_ruff_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_ruff_command, ["--service", "backend"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "backend" in cmd

    def test_command_combined_options(self):
        """n7 dkc pyrf src/ --fix --diff -v"""
        with patch("n7.commands.docker.format.docker_ruff_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_ruff_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(
                    docker_ruff_command,
                    [
                        "--fix",
                        "--diff",
                        "-v",
                        "src/",
                    ],
                )

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--fix" in cmd
                assert "--diff" in cmd
                assert "--verbose" in cmd
                assert cmd[-1] == "src/"

    def test_command_with_rules_options(self):
        """n7 dkc pyrf with rules options"""
        with patch("n7.commands.docker.format.docker_ruff_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_ruff_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(
                    docker_ruff_command,
                    [
                        "--select",
                        "E,F",
                        "--ignore",
                        "E501",
                        "--extend-select",
                        "B",
                    ],
                )

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--select" in cmd
                assert "E,F" in cmd
                assert "--ignore" in cmd
                assert "E501" in cmd
                assert "--extend-select" in cmd
                assert "B" in cmd



class MypyCommandBuilder:
    """Build other commands mypy"""

    def __init__(self):
        self.base_cmd = "mypy"

    def build(
        self,
        path: str | None = None,
        strict: bool = False,
        check_untyped_defs: bool = False,
        disallow_untyped_defs: bool = False,
        disallow_any_expr: bool = False,
        no_implicit_optional: bool = False,
        warn_return_any: bool = False,
        warn_unused_ignores: bool = False,
        show_error_codes: bool = False,
        pretty: bool = False,
        no_error_summary: bool = False,
        config_file: str | None = None,
        python_version: str | None = None,
        exclude: str | None = None,
        verbose: bool = False,
        quiet: bool = False,
    ) -> tuple[str, ...]:
        """Build a mypy command with arguments"""
        cmd: tuple[str, ...] = (self.base_cmd,)

        if strict:
            cmd = (*cmd, "--strict")
        if check_untyped_defs:
            cmd = (*cmd, "--check-untyped-defs")
        if disallow_untyped_defs:
            cmd = (*cmd, "--disallow-untyped-defs")
        if disallow_any_expr:
            cmd = (*cmd, "--disallow-any-expr")
        if no_implicit_optional:
            cmd = (*cmd, "--no-implicit-optional")
        if warn_return_any:
            cmd = (*cmd, "--warn-return-any")
        if warn_unused_ignores:
            cmd = (*cmd, "--warn-unused-ignores")
        if show_error_codes:
            cmd = (*cmd, "--show-error-codes")
        if pretty:
            cmd = (*cmd, "--pretty")
        if no_error_summary:
            cmd = (*cmd, "--no-error-summary")
        if config_file:
            cmd = (*cmd, "--config-file", config_file)
        if python_version:
            cmd = (*cmd, "--python-version", python_version)
        if exclude:
            cmd = (*cmd, "--exclude", exclude)
        if verbose:
            cmd = (*cmd, "--verbose")
        if quiet:
            cmd = (*cmd, "--quiet")
        if path:
            cmd = (*cmd, path)

        return cmd



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.local.format.local_pyqa_command import local_pyqa_command

runner = CliRunner()


class TestLocalPyqaCommand:
    """Tests pour la commande pyqa local"""

    def test_command_without_options(self):
        """command n7 pyqa -> black --check, ruff check, mypy"""
        with patch("n7.commands.local.format.local_pyqa_command.subprocess.run") as mock_run:
            result = runner.invoke(local_pyqa_command, [])

            assert result.exit_code == 0
            assert mock_run.call_count == 3

            # Vérifier les commandes appelées
            calls = mock_run.call_args_list
            black_cmd = calls[0][0][0]
            ruff_cmd = calls[1][0][0]
            mypy_cmd = calls[2][0][0]

            assert black_cmd == ("black", "--check", ".")
            assert ruff_cmd == ("ruff", "check", ".")
            assert mypy_cmd == ("mypy", ".")

    def test_command_with_path(self):
        """command n7 pyqa src/"""
        with patch("n7.commands.local.format.local_pyqa_command.subprocess.run") as mock_run:
            result = runner.invoke(local_pyqa_command, ["src/"])

            assert result.exit_code == 0
            assert mock_run.call_count == 3

            calls = mock_run.call_args_list
            black_cmd = calls[0][0][0]
            ruff_cmd = calls[1][0][0]
            mypy_cmd = calls[2][0][0]

            assert black_cmd == ("black", "--check", "src/")
            assert ruff_cmd == ("ruff", "check", "src/")
            assert mypy_cmd == ("mypy", "src/")

    def test_command_with_fix(self):
        """command n7 pyqa --fix -> black (format), ruff check --fix, mypy"""
        with patch("n7.commands.local.format.local_pyqa_command.subprocess.run") as mock_run:
            result = runner.invoke(local_pyqa_command, ["--fix"])

            assert result.exit_code == 0
            assert mock_run.call_count == 3

            calls = mock_run.call_args_list
            black_cmd = calls[0][0][0]
            ruff_cmd = calls[1][0][0]
            mypy_cmd = calls[2][0][0]

            # Black sans --check (mode format)
            assert black_cmd == ("black", ".")
            # Ruff avec --fix
            assert ruff_cmd == ("ruff", "check", "--fix", ".")
            # Mypy reste inchangé
            assert mypy_cmd == ("mypy", ".")

    def test_command_with_fix_and_path(self):
        """command n7 pyqa --fix src/"""
        with patch("n7.commands.local.format.local_pyqa_command.subprocess.run") as mock_run:
            result = runner.invoke(local_pyqa_command, ["--fix", "src/"])

            assert result.exit_code == 0
            assert mock_run.call_count == 3

            calls = mock_run.call_args_list
            black_cmd = calls[0][0][0]
            ruff_cmd = calls[1][0][0]
            mypy_cmd = calls[2][0][0]

            assert black_cmd == ("black", "src/")
            assert ruff_cmd == ("ruff", "check", "--fix", "src/")
            assert mypy_cmd == ("mypy", "src/")

    def test_commands_executed_sequentially(self):
        """Vérifie que les commandes sont exécutées dans l'ordre: black, ruff, mypy"""
        with patch("n7.commands.local.format.local_pyqa_command.subprocess.run") as mock_run:
            runner.invoke(local_pyqa_command, [])

            calls = mock_run.call_args_list
            assert len(calls) == 3
            assert calls[0][0][0][0] == "black"
            assert calls[1][0][0][0] == "ruff"
            assert calls[2][0][0][0] == "mypy"



import subprocess
from typing import List

import typer

from n7.commands.builders import DockerComposeCommandBuilder
from n7.commands.resolver import DockerFileResolver

docker_uv_command = typer.Typer(
    help="Run uv command with any arguments in Docker container",
    invoke_without_command=True,
    no_args_is_help=False,
)


@docker_uv_command.callback()
def run(
    args: List[str] = typer.Argument(None, help="Arguments to pass to uv command"),
    service: str = typer.Option(None, "--service", "-s", help="Service docker"),
):
    """Run uv command with any arguments in Docker container"""
    # Récupérer tous les arguments passés à uv
    uv_args = args if args else []

    # Résolution des fichiers docker
    resolver = DockerFileResolver()
    config_docker = resolver.resolve()

    # Service à utiliser
    target_service = service or config_docker["default_service"] or "api"

    # Build command uv
    uv_cmd = ("uv", *uv_args)

    # Build commande docker compose
    docker_builder = DockerComposeCommandBuilder(
        env_file=config_docker["env_file"] or ".env",
        compose_file=config_docker["compose_file"] or "docker-compose.yml",
    )

    # Build docker command
    docker_cmd = docker_builder.build_exec(service=target_service, cmd=uv_cmd)

    # Exécution dans le container
    subprocess.run(docker_cmd, check=True)



from n7.commands.builders import BlackCommandBuilder


class TestBlackCommandBuilder:
    def setup_method(self):
        self.builder = BlackCommandBuilder()

    def test_instance_builder(self):
        """test instance builder"""
        assert self.builder is not None
        assert self.builder.base_cmd == "black"

    def test_build_command_without_all(self):
        """command black only"""
        res = self.builder.build()
        assert res == ("black",)

    def test_build_command_with_path(self):
        """command black with path"""
        res = self.builder.build(path="my/path")
        assert res == ("black", "my/path")

    def test_build_with_check(self):
        """command black --check"""
        result = self.builder.build(check=True)
        assert result == ("black", "--check")

    def test_build_with_diff(self):
        """command black --diff"""
        result = self.builder.build(diff=True)
        assert result == ("black", "--diff")

    def test_build_with_color(self):
        """command black --color"""
        result = self.builder.build(color=True)
        assert result == ("black", "--color")

    def test_build_with_no_color(self):
        """command black --no-color"""
        result = self.builder.build(no_color=True)
        assert result == ("black", "--no-color")

    def test_build_with_line_length(self):
        """command black --line-length 100"""
        result = self.builder.build(line_length="100")
        assert result == ("black", "--line-length", "100")

    def test_build_with_line_length_default(self):
        """command black --line-length 88"""
        result = self.builder.build(line_length="88")
        assert result == ("black", "--line-length", "88")

    def test_build_with_target_version(self):
        """command black --target-version py313"""
        result = self.builder.build(target_version="py313")
        assert result == ("black", "--target-version", "py313")

    def test_build_with_skip_string_normalization(self):
        """command black --skip-string-normalization"""
        result = self.builder.build(skip_string_normalization=True)
        assert result == ("black", "--skip-string-normalization")

    def test_build_with_quiet(self):
        """command black --quiet"""
        result = self.builder.build(quiet=True)
        assert result == ("black", "--quiet")

    def test_build_with_verbose(self):
        """command black --verbose"""
        result = self.builder.build(verbose=True)
        assert result == ("black", "--verbose")

    def test_build_with_fast(self):
        """command black --fast"""
        result = self.builder.build(fast=True)
        assert result == ("black", "--fast")

    def test_build_with_exclude(self):
        """command black --exclude pattern"""
        result = self.builder.build(exclude="venv")
        assert result == ("black", "--exclude", "venv")

    def test_build_with_include(self):
        """command black --include pattern"""
        result = self.builder.build(include="\\.pyi?$")
        assert result == ("black", "--include", "\\.pyi?$")

    def test_build_with_multiple_options(self):
        """command black with many options"""
        result = self.builder.build(
            path="src/",
            check=True,
            diff=True,
            verbose=True,
        )
        assert result == ("black", "--check", "--diff", "--verbose", "src/")

    def test_build_with_formatting_options(self):
        """command black with formatting options"""
        result = self.builder.build(
            line_length="100",
            target_version="py313",
            skip_string_normalization=True,
        )
        assert result == (
            "black",
            "--line-length",
            "100",
            "--target-version",
            "py313",
            "--skip-string-normalization",
        )

    def test_path_always_last(self):
        """This path always last"""
        result = self.builder.build(
            path="src/",
            check=True,
            line_length="100",
            verbose=True,
        )
        assert result[-1] == "src/"

    def test_build_full_command(self):
        """command black with all common options"""
        result = self.builder.build(
            path=".",
            check=True,
            diff=True,
            color=True,
            line_length="100",
            target_version="py313",
            verbose=True,
        )
        assert result == (
            "black",
            "--check",
            "--diff",
            "--color",
            "--line-length",
            "100",
            "--target-version",
            "py313",
            "--verbose",
            ".",
        )

    def test_color_and_no_color_not_both(self):
        """test that color takes precedence over no_color if both are set"""
        # Dans l'implémentation, on gérera ce cas en donnant la priorité à color
        result = self.builder.build(color=True, no_color=True)
        assert "--color" in result
        assert "--no-color" not in result



import subprocess

import typer

from n7.commands.builders import DockerComposeCommandBuilder
from n7.commands.resolver import DockerFileResolver

connect_shell_command = typer.Typer(help="Start shell in container")


@connect_shell_command.callback(invoke_without_command=True)
def shell_command(
    no_bash: bool = typer.Option(False, "--no-bash", help="no shell bash, shell becomes 'sh'"),
    service: str = typer.Option(None, "--service", "-s", help="Service docker"),
):
    resolver = DockerFileResolver()
    config = resolver.resolve()

    target_service = service or config["default_service"] or "api"

    # Build commande docker compose
    docker_builder = DockerComposeCommandBuilder(
        env_file=config["env_file"] or ".env",
        compose_file=config["compose_file"] or "docker-compose.yml",
    )

    command = ("bash",)
    if no_bash:
        command = ("sh",)

    cmd = docker_builder.build_exec(service=target_service, cmd=command, disable_tty=False)

    subprocess.run(cmd, check=True)



import subprocess

import typer

from n7.commands.builders import (
    BlackCommandBuilder,
    MypyCommandBuilder,
    PytestCommandBuilder,
    RuffCommandBuilder,
)

local_pycheck_command = typer.Typer(help="Run Python checks (Pytest, Black, Ruff, Mypy)")


@local_pycheck_command.callback(invoke_without_command=True)
def run(
    ctx: typer.Context,
    path: str = typer.Argument(None, help="Path to check (file or directory)"),
    fix: bool = typer.Option(False, "--fix", help="Apply fixes (Black format, Ruff --fix)"),
    verbose: bool = typer.Option(False, "-v", "--verbose", help="Verbose pytest output"),
    stop_first: bool = typer.Option(
        False, "-x", "--stop-first", help="Stop pytest at first failure"
    ),
    filter_tests: str = typer.Option(None, "-k", help="Filter pytest by test name expression"),
    parallel: bool = typer.Option(False, "--parallel", help="Run tools in parallel"),
):
    """Run Python checks: Pytest, Black, Ruff, and Mypy"""
    if ctx.invoked_subcommand is not None:
        return

    # Utiliser "." comme path par défaut si aucun path n'est spécifié
    target_path = path or "."

    # Build Pytest command
    pytest_builder = PytestCommandBuilder()
    pytest_cmd = pytest_builder.build(
        path=path, verbose=verbose, stop_first=stop_first, filter_tests=filter_tests
    )

    # Build Black command
    black_builder = BlackCommandBuilder()
    if fix:
        # Mode format: pas de --check
        black_cmd = black_builder.build(path=target_path)
    else:
        # Mode check: avec --check
        black_cmd = black_builder.build(path=target_path, check=True)

    # Build Ruff command
    ruff_builder = RuffCommandBuilder()
    ruff_cmd = ruff_builder.build(path=target_path, fix=fix)

    # Build Mypy command
    mypy_builder = MypyCommandBuilder()
    mypy_cmd = mypy_builder.build(path=target_path)

    if parallel:
        # Exécution parallèle
        processes = []
        processes.append(subprocess.Popen(pytest_cmd))
        processes.append(subprocess.Popen(black_cmd))
        processes.append(subprocess.Popen(ruff_cmd))
        processes.append(subprocess.Popen(mypy_cmd))

        # Attendre que tous les processus se terminent
        exit_codes = [p.wait() for p in processes]

        # Si un processus a échoué, lever une erreur
        if any(code != 0 for code in exit_codes):
            raise subprocess.CalledProcessError(max(exit_codes), "parallel execution")
    else:
        # Exécution séquentielle
        subprocess.run(pytest_cmd, check=True)
        subprocess.run(black_cmd, check=True)
        subprocess.run(ruff_cmd, check=True)
        subprocess.run(mypy_cmd, check=True)



import subprocess

import typer

from n7.commands.builders import PytestCommandBuilder

local_pytest_command = typer.Typer(help="Tests pytest en local")


@local_pytest_command.callback(invoke_without_command=True)
def run(
    ctx: typer.Context,
    path: str = typer.Argument(None, help="Path of test"),
    verbose: bool = typer.Option(False, "-v", help="Mode verbose"),
    stop_first: bool = typer.Option(False, "-x", help="Stop first failure"),
    workers: str = typer.Option(None, "-n", help="number of workers (auto, 2, 4...)"),
    cov: str = typer.Option(None, "--cov", help="App for coverage"),
    cov_report: str = typer.Option(None, "--cov-report", help="Type rapport (html, term-missing)"),
    last_failed: bool = typer.Option(False, "--lf", help="Re try tests fails"),
    failed_first: bool = typer.Option(False, "--ff", help="Tests fails first time"),
    create_db: bool = typer.Option(False, "--create-db", help="Force create DB (test)"),
    migrations: bool = typer.Option(False, "--migrations", help="Force migrations (test)"),
):
    """Lance les tests pytest en local"""
    if ctx.invoked_subcommand is not None:
        return

    # Build commande pytest
    pytest_builder = PytestCommandBuilder()
    cmd = pytest_builder.build(
        path=path,
        verbose=verbose,
        stop_first=stop_first,
        workers=workers,
        cov=cov,
        cov_report=cov_report,
        last_failed=last_failed,
        failed_first=failed_first,
        create_db=create_db,
        migrations=migrations,
    )

    # Exécution
    subprocess.run(cmd, check=True)



class BlackCommandBuilder:
    """Build other commands black"""

    def __init__(self):
        self.base_cmd = "black"

    def build(
        self,
        path: str | None = None,
        check: bool = False,
        diff: bool = False,
        color: bool = False,
        no_color: bool = False,
        line_length: str | None = None,
        target_version: str | None = None,
        skip_string_normalization: bool = False,
        quiet: bool = False,
        verbose: bool = False,
        fast: bool = False,
        exclude: str | None = None,
        include: str | None = None,
    ) -> tuple[str, ...]:
        """Build a black command with arguments"""
        cmd: tuple[str, ...] = (self.base_cmd,)

        if check:
            cmd = (*cmd, "--check")
        if diff:
            cmd = (*cmd, "--diff")
        # color takes precedence over no_color
        if color:
            cmd = (*cmd, "--color")
        elif no_color:
            cmd = (*cmd, "--no-color")
        if line_length:
            cmd = (*cmd, "--line-length", line_length)
        if target_version:
            cmd = (*cmd, "--target-version", target_version)
        if skip_string_normalization:
            cmd = (*cmd, "--skip-string-normalization")
        if quiet:
            cmd = (*cmd, "--quiet")
        if verbose:
            cmd = (*cmd, "--verbose")
        if fast:
            cmd = (*cmd, "--fast")
        if exclude:
            cmd = (*cmd, "--exclude", exclude)
        if include:
            cmd = (*cmd, "--include", include)
        if path:
            cmd = (*cmd, path)

        return cmd



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.docker.format.docker_black_command import docker_black_command

runner = CliRunner()


class TestDockerBlackCommand:
    """Tests pour la commande docker black"""

    def test_command_without_options(self):
        """n7 dkc pybl -> black simple dans container"""
        with patch("n7.commands.docker.format.docker_black_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_black_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_black_command, [])

                assert result.exit_code == 0
                mock_run.assert_called_once()
                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "api",
                    "black",
                )

    def test_command_with_path(self):
        """n7 dkc pybl src/"""
        with patch("n7.commands.docker.format.docker_black_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_black_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_black_command, ["src/"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert cmd[-1] == "src/"

    def test_command_with_check(self):
        """n7 dkc pybl --check"""
        with patch("n7.commands.docker.format.docker_black_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_black_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_black_command, ["--check"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--check" in cmd

    def test_command_with_diff(self):
        """n7 dkc pybl --diff"""
        with patch("n7.commands.docker.format.docker_black_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_black_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_black_command, ["--diff"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--diff" in cmd

    def test_command_with_line_length(self):
        """n7 dkc pybl --line-length 100"""
        with patch("n7.commands.docker.format.docker_black_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_black_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_black_command, ["--line-length", "100"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--line-length" in cmd
                assert "100" in cmd

    def test_command_with_target_version(self):
        """n7 dkc pybl --target-version py313"""
        with patch("n7.commands.docker.format.docker_black_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_black_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_black_command, ["--target-version", "py313"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--target-version" in cmd
                assert "py313" in cmd

    def test_command_with_verbose(self):
        """n7 dkc pybl -v"""
        with patch("n7.commands.docker.format.docker_black_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_black_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_black_command, ["-v"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--verbose" in cmd

    def test_command_with_custom_service(self):
        """n7 dkc pybl --service backend"""
        with patch("n7.commands.docker.format.docker_black_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_black_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_black_command, ["--service", "backend"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "backend" in cmd

    def test_command_combined_options(self):
        """n7 dkc pybl src/ --check --diff -v"""
        with patch("n7.commands.docker.format.docker_black_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_black_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(
                    docker_black_command,
                    [
                        "--check",
                        "--diff",
                        "-v",
                        "src/",
                    ],
                )

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--check" in cmd
                assert "--diff" in cmd
                assert "--verbose" in cmd
                assert cmd[-1] == "src/"

    def test_command_with_formatting_options(self):
        """n7 dkc pybl with formatting options"""
        with patch("n7.commands.docker.format.docker_black_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_black_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(
                    docker_black_command,
                    [
                        "--line-length",
                        "100",
                        "--target-version",
                        "py313",
                        "--skip-string-normalization",
                    ],
                )

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--line-length" in cmd
                assert "100" in cmd
                assert "--target-version" in cmd
                assert "py313" in cmd
                assert "--skip-string-normalization" in cmd



import subprocess

import typer

from n7.commands.builders.docker_compose_command_builder import DockerComposeCommandBuilder
from n7.commands.builders.pytest_command_builder import PytestCommandBuilder
from n7.commands.resolver.docker_file_resolver import DockerFileResolver

docker_pytest_command = typer.Typer(help="Tests pytest in container docker python")


@docker_pytest_command.callback(invoke_without_command=True)
def run(
    ctx: typer.Context,
    path: str = typer.Argument(None, help="Path of test"),
    verbose: bool = typer.Option(False, "-v", help="Mode verbose"),
    stop_first: bool = typer.Option(False, "-x", help="Stop on first failure"),
    workers: str = typer.Option(None, "-n", help="Number of workers (auto, 2, 4...)"),
    cov: str = typer.Option(None, "--cov", help="App (module python) for coverage"),
    cov_report: str = typer.Option(None, "--cov-report", help="Type rapport (html, term-missing)"),
    last_failed: bool = typer.Option(False, "--lf", help="Re try tests fails"),
    failed_first: bool = typer.Option(False, "--ff", help="Tests fails first time"),
    create_db: bool = typer.Option(False, "--create-db", help="Force create DB (test)"),
    migrations: bool = typer.Option(False, "--migrations", help="Force migrations (test)"),
    service: str = typer.Option(None, "--service", "-s", help="Service docker"),
):
    """Lance les tests pytest dans le container"""
    if ctx.invoked_subcommand is not None:
        return

    # résolution des fichiers docker
    resolver = DockerFileResolver()
    config = resolver.resolve()

    # service à utiliser
    target_service = service or config["default_service"] or "api"

    # build commande pytest
    pytest_builder = PytestCommandBuilder()
    pytest_cmd = pytest_builder.build(
        path=path,
        verbose=verbose,
        stop_first=stop_first,
        workers=workers,
        cov=cov,
        cov_report=cov_report,
        last_failed=last_failed,
        failed_first=failed_first,
        create_db=create_db,
        migrations=migrations,
    )

    # build commande docker compose
    docker_builder = DockerComposeCommandBuilder(
        env_file=config["env_file"] or ".env",
        compose_file=config["compose_file"] or "docker-compose.yml",
    )
    cmd = docker_builder.build_exec(service=target_service, cmd=pytest_cmd)

    # execution
    subprocess.run(cmd, check=True)



class RuffCommandBuilder:
    """Build other commands ruff"""

    def __init__(self):
        self.base_cmd = "ruff"

    def build(
        self,
        path: str | None = None,
        fix: bool = False,
        diff: bool = False,
        watch: bool = False,
        fix_only: bool = False,
        unsafe_fixes: bool = False,
        show_fixes: bool = False,
        output_format: str | None = None,
        config: str | None = None,
        select: str | None = None,
        ignore: str | None = None,
        exclude: str | None = None,
        extend_select: str | None = None,
        quiet: bool = False,
        verbose: bool = False,
    ) -> tuple[str, ...]:
        """Build a ruff command with arguments"""
        cmd: tuple[str, ...] = (self.base_cmd, "check")

        if fix:
            cmd = (*cmd, "--fix")
        if diff:
            cmd = (*cmd, "--diff")
        if watch:
            cmd = (*cmd, "--watch")
        if fix_only:
            cmd = (*cmd, "--fix-only")
        if unsafe_fixes:
            cmd = (*cmd, "--unsafe-fixes")
        if show_fixes:
            cmd = (*cmd, "--show-fixes")
        if output_format:
            cmd = (*cmd, "--output-format", output_format)
        if config:
            cmd = (*cmd, "--config", config)
        if select:
            cmd = (*cmd, "--select", select)
        if ignore:
            cmd = (*cmd, "--ignore", ignore)
        if exclude:
            cmd = (*cmd, "--exclude", exclude)
        if extend_select:
            cmd = (*cmd, "--extend-select", extend_select)
        if quiet:
            cmd = (*cmd, "--quiet")
        if verbose:
            cmd = (*cmd, "--verbose")
        if path:
            cmd = (*cmd, path)

        return cmd



from n7.commands.resolver import DockerFileResolver


class TestDockerFileResolver:
    """Tests pour DockerFileResolver"""

    def test_find_compose_file_at_root(self, tmp_path):
        """Trouve docker-compose.yml à la racine"""
        (tmp_path / "docker-compose.yml").touch()

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.find_compose_file()

        assert result == "docker-compose.yml"

    def test_find_compose_file_yaml_extension(self, tmp_path):
        """Trouve docker-compose.yaml à la racine"""
        (tmp_path / "docker-compose.yaml").touch()

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.find_compose_file()

        assert result == "docker-compose.yaml"

    def test_find_compose_file_short_name(self, tmp_path):
        """Trouve compose.yml à la racine"""
        (tmp_path / "compose.yml").touch()

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.find_compose_file()

        assert result == "compose.yml"

    def test_find_compose_file_with_env(self, tmp_path):
        """Trouve docker-compose.dev.yml"""
        (tmp_path / "docker-compose.dev.yml").touch()

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.find_compose_file()

        assert result == "docker-compose.dev.yml"

    def test_find_compose_file_in_docker_folder(self, tmp_path):
        """Trouve compose.yml dans docker/"""
        (tmp_path / "docker").mkdir()
        (tmp_path / "docker" / "compose.yml").touch()

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.find_compose_file()

        assert result == "docker/compose.yml"

    def test_find_compose_file_priority_root_over_folder(self, tmp_path):
        """Priorité racine sur docker/"""
        (tmp_path / "docker-compose.yml").touch()
        (tmp_path / "docker").mkdir()
        (tmp_path / "docker" / "compose.yml").touch()

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.find_compose_file()

        assert result == "docker-compose.yml"

    def test_find_compose_file_returns_none_when_not_found(self, tmp_path):
        """Retourne None si aucun fichier trouvé"""
        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.find_compose_file()

        assert result is None

    def test_find_env_file_at_root(self, tmp_path):
        """Trouve .env à la racine"""
        (tmp_path / ".env").touch()

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.find_env_file()

        assert result == ".env"

    def test_find_env_file_with_suffix(self, tmp_path):
        """Trouve .env.local"""
        (tmp_path / ".env.local").touch()

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.find_env_file()

        assert result == ".env.local"

    def test_find_env_file_with_suffix_api(self, tmp_path):
        """Trouve .env.local"""
        (tmp_path / ".env.api").touch()

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.find_env_file()

        assert result == ".env.api"

    def test_find_env_file_in_api_folder(self, tmp_path):
        """Trouve .env dans api/"""
        (tmp_path / "api").mkdir()
        (tmp_path / "api" / ".env").touch()

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.find_env_file()

        assert result == "api/.env"

    def test_find_env_file_priority_root_over_folder(self, tmp_path):
        """Priorité racine sur dossiers"""
        (tmp_path / ".env").touch()
        (tmp_path / "api").mkdir()
        (tmp_path / "api" / ".env").touch()

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.find_env_file()

        assert result == ".env"

    def test_find_env_file_returns_none_when_not_found(self, tmp_path):
        """Retourne None si aucun fichier trouvé"""
        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.find_env_file()

        assert result is None

    def test_resolve_uses_config_override(self, tmp_path):
        """Utilise les valeurs de n7.yaml si présent"""
        config_content = """
        docker:
          path_env_file: .env.prod
          path_compose_file: docker-compose.prod.yml
        """
        (tmp_path / "n7.yaml").write_text(config_content)
        (tmp_path / ".env").touch()
        (tmp_path / "docker-compose.yml").touch()

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.resolve()

        assert result["env_file"] == ".env.prod"
        assert result["compose_file"] == "docker-compose.prod.yml"

    def test_resolve_uses_auto_detection(self, tmp_path):
        """Utilise la détection auto si pas de n7.yaml"""
        (tmp_path / ".env.local").touch()
        (tmp_path / "docker-compose.dev.yml").touch()

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.resolve()

        assert result["env_file"] == ".env.local"
        assert result["compose_file"] == "docker-compose.dev.yml"

    def test_resolve_returns_default_service(self, tmp_path):
        """Retourne le default_service de la config"""
        config_content = """
        docker:
          default_service: worker
        """
        (tmp_path / "n7.yaml").write_text(config_content)

        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.resolve()

        assert result["default_service"] == "worker"

    def test_resolve_default_service_none_when_not_set(self, tmp_path):
        """default_service est None si pas défini"""
        resolver = DockerFileResolver(base_path=tmp_path)
        result = resolver.resolve()

        assert result["default_service"] is None



from n7.commands.builders import DockerComposeCommandBuilder


class TestDockerComposeCommandBuilder:
    def setup_method(self):
        self.builder = DockerComposeCommandBuilder(
            env_file=".env",
            compose_file="docker-compose.yml",
        )

    def test_instance(self):
        """test docker compose instance"""
        assert self.builder is not None

    def test_build_exec_simple(self):
        """command docker compose exec service cmd"""
        result = self.builder.build_exec(
            service="api",
            cmd=("bash",),
        )
        assert result == (
            "docker",
            "compose",
            "--env-file",
            ".env",
            "-f",
            "docker-compose.yml",
            "exec",
            "api",
            "bash",
        )

    def test_build_exec_with_command_options(self):
        """command docker compose exec with options"""
        result = self.builder.build_exec(
            service="api",
            cmd=("pytest", "-v", "-n", "auto"),
        )
        assert result == (
            "docker",
            "compose",
            "--env-file",
            ".env",
            "-f",
            "docker-compose.yml",
            "exec",
            "api",
            "pytest",
            "-v",
            "-n",
            "auto",
        )

    def test_build_exec_with_disable_tty(self):
        """command docker compose exec with TTY disabled"""
        result = self.builder.build_exec(
            service="api",
            cmd=("pytest",),
            disable_tty=True,
        )
        assert result == (
            "docker",
            "compose",
            "--env-file",
            ".env",
            "-f",
            "docker-compose.yml",
            "exec",
            "-T",
            "api",
            "pytest",
        )

    def test_build_up(self):
        """command docker compose up -d"""
        result = self.builder.build_up()
        assert result == (
            "docker",
            "compose",
            "--env-file",
            ".env",
            "-f",
            "docker-compose.yml",
            "up",
            "-d",
        )

    def test_build_up_without_detach(self):
        """command docker compose up sans -d"""
        result = self.builder.build_up(detach=False)
        assert result == (
            "docker",
            "compose",
            "--env-file",
            ".env",
            "-f",
            "docker-compose.yml",
            "up",
        )

    def test_build_down(self):
        """command docker compose down"""
        result = self.builder.build_down()
        assert result == (
            "docker",
            "compose",
            "--env-file",
            ".env",
            "-f",
            "docker-compose.yml",
            "down",
        )

    def test_build_down_with_volumes(self):
        """command docker compose down -v"""
        result = self.builder.build_down(volumes=True)
        assert result == (
            "docker",
            "compose",
            "--env-file",
            ".env",
            "-f",
            "docker-compose.yml",
            "down",
            "-v",
        )

    def test_build_logs(self):
        """command docker compose logs service"""
        result = self.builder.build_logs(service="api")
        assert result == (
            "docker",
            "compose",
            "--env-file",
            ".env",
            "-f",
            "docker-compose.yml",
            "logs",
            "api",
        )

    def test_build_logs_with_follow(self):
        """command docker compose logs -f service"""
        result = self.builder.build_logs(service="api", follow=True)
        assert result == (
            "docker",
            "compose",
            "--env-file",
            ".env",
            "-f",
            "docker-compose.yml",
            "logs",
            "-f",
            "api",
        )

    def test_custom_files(self):
        """Builder avec fichiers custom"""
        builder = DockerComposeCommandBuilder(
            env_file=".env.prod",
            compose_file="docker-compose.prod.yml",
        )
        result = builder.build_up()
        assert result == (
            "docker",
            "compose",
            "--env-file",
            ".env.prod",
            "-f",
            "docker-compose.prod.yml",
            "up",
            "-d",
        )



class PytestCommandBuilder:
    """Build other commands pytest"""

    def __init__(self):
        self.base_cmd = "pytest"

    def build(
        self,
        path: str | None = None,
        verbose: bool = False,
        stop_first: bool = False,
        filter_tests: str | None = None,
        workers: str | None = None,
        cov: str | None = None,
        cov_report: str | None = None,
        last_failed: bool = False,
        failed_first: bool = False,
        create_db: bool = False,
        migrations: bool = False,
    ) -> tuple[str, ...]:
        """Build a pytest command with arguments"""
        cmd: tuple[str, ...] = (self.base_cmd,)

        if verbose:
            cmd = (*cmd, "-v")
        if stop_first:
            cmd = (*cmd, "-x")
        if filter_tests:
            cmd = (*cmd, "-k", filter_tests)
        if workers:
            cmd = (*cmd, "-n", workers)
        if cov:
            cmd = (*cmd, f"--cov={cov}")
        if cov_report:
            cmd = (*cmd, f"--cov-report={cov_report}")
        if last_failed:
            cmd = (*cmd, "--lf")
        if failed_first:
            cmd = (*cmd, "--ff")
        if create_db:
            cmd = (*cmd, "--create-db")
        if migrations:
            cmd = (*cmd, "--migrations")
        if path:
            cmd = (*cmd, path)

        return cmd



import subprocess

import typer

from n7.commands.builders import RuffCommandBuilder

local_ruff_command = typer.Typer(help="Lint Python code with Ruff")


@local_ruff_command.callback(invoke_without_command=True)
def run(
    ctx: typer.Context,
    path: str = typer.Argument(None, help="Path to lint (file or directory)"),
    fix: bool = typer.Option(False, "--fix", help="Fix linting errors automatically"),
    diff: bool = typer.Option(False, "--diff", help="Show diff of fixes"),
    watch: bool = typer.Option(False, "--watch", help="Watch mode"),
    fix_only: bool = typer.Option(False, "--fix-only", help="Apply fixes without reporting"),
    unsafe_fixes: bool = typer.Option(False, "--unsafe-fixes", help="Apply unsafe fixes"),
    show_fixes: bool = typer.Option(False, "--show-fixes", help="Show available fixes"),
    output_format: str = typer.Option(
        None, "--output-format", help="Output format (text, json, etc.)"
    ),
    config: str = typer.Option(None, "--config", help="Config file path"),
    select: str = typer.Option(None, "--select", help="Select rules (e.g., E,F)"),
    ignore: str = typer.Option(None, "--ignore", help="Ignore rules (e.g., E501)"),
    exclude: str = typer.Option(None, "--exclude", help="Exclude pattern"),
    extend_select: str = typer.Option(None, "--extend-select", help="Extend selected rules"),
    quiet: bool = typer.Option(False, "-q", "--quiet", help="Quiet mode"),
    verbose: bool = typer.Option(False, "-v", "--verbose", help="Verbose mode"),
):
    """Lint Python code with Ruff"""
    if ctx.invoked_subcommand is not None:
        return

    # Build commande ruff
    ruff_builder = RuffCommandBuilder()
    cmd = ruff_builder.build(
        path=path,
        fix=fix,
        diff=diff,
        watch=watch,
        fix_only=fix_only,
        unsafe_fixes=unsafe_fixes,
        show_fixes=show_fixes,
        output_format=output_format,
        config=config,
        select=select,
        ignore=ignore,
        exclude=exclude,
        extend_select=extend_select,
        quiet=quiet,
        verbose=verbose,
    )

    # Exécution
    subprocess.run(cmd, check=True)



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.docker.container.logs_command import logs_command

runner = CliRunner()


class TestLogsCommand:
    """Tests pour la commande docker compose logs"""

    def test_logs_without_service(self):
        """n7 dkc logs -> docker compose logs (tous les services)"""
        with patch("n7.commands.docker.container.logs_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.logs_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(logs_command, [])

                assert result.exit_code == 0
                mock_run.assert_called_once()
                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "logs",
                    "",
                )

    def test_logs_with_service(self):
        """n7 dkc logs api"""
        with patch("n7.commands.docker.container.logs_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.logs_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(logs_command, ["api"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "api" in cmd
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "logs",
                    "api",
                )

    def test_logs_with_multiple_services(self):
        """n7 dkc logs worker"""
        with patch("n7.commands.docker.container.logs_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.logs_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(logs_command, ["worker"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "worker" in cmd

    def test_logs_with_follow(self):
        """n7 dkc logs --follow"""
        with patch("n7.commands.docker.container.logs_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.logs_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(logs_command, ["--follow"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "-f" in cmd
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "logs",
                    "-f",
                    "",
                )

    def test_logs_with_follow_short_option(self):
        """n7 dkc logs -f"""
        with patch("n7.commands.docker.container.logs_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.logs_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(logs_command, ["-f"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "-f" in cmd

    def test_logs_with_service_and_follow(self):
        """n7 dkc logs --follow api"""
        with patch("n7.commands.docker.container.logs_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.logs_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(logs_command, ["--follow", "api"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "api" in cmd
                assert "-f" in cmd
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "logs",
                    "-f",
                    "api",
                )

    def test_logs_with_service_and_follow_short_option(self):
        """n7 dkc logs -f api"""
        with patch("n7.commands.docker.container.logs_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.logs_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(logs_command, ["-f", "api"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "api" in cmd
                assert "-f" in cmd

    def test_logs_custom_files(self):
        """n7 dkc logs worker avec fichiers custom via n7.yaml"""
        with patch("n7.commands.docker.container.logs_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.logs_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env.prod",
                    "compose_file": "docker-compose.prod.yml",
                    "default_service": None,
                }

                result = runner.invoke(logs_command, ["worker"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert ".env.prod" in cmd
                assert "docker-compose.prod.yml" in cmd
                assert "worker" in cmd



import subprocess

import typer

from n7.commands.builders import DockerComposeCommandBuilder
from n7.commands.resolver import DockerFileResolver

connect_postgres_command = typer.Typer(help="Connect to PostgreSQL database in container")


@connect_postgres_command.callback(invoke_without_command=True)
def postgres_command(
    service: str = typer.Option(None, "--service", "-s", help="Service docker"),
    user: str = typer.Option(None, "--user", "-u", help="PostgreSQL user"),
    database: str = typer.Option(None, "--database", "-d", help="PostgreSQL database"),
):
    """Connect to PostgreSQL database using psql"""
    resolver = DockerFileResolver()
    config = resolver.resolve()

    target_service = service or "db"

    # Build commande docker compose
    docker_builder = DockerComposeCommandBuilder(
        env_file=config["env_file"] or ".env",
        compose_file=config["compose_file"] or "docker-compose.yml",
    )

    # Build psql command
    command = ["psql"]
    if user:
        command.extend(["-U", user])
    if database:
        command.extend(["-d", database])

    cmd = docker_builder.build_exec(service=target_service, cmd=tuple(command), disable_tty=False)

    subprocess.run(cmd, check=True)



import subprocess

import typer

from n7.commands.builders import BlackCommandBuilder

local_black_command = typer.Typer(help="Format Python code with Black")


@local_black_command.callback(invoke_without_command=True)
def run(
    ctx: typer.Context,
    path: str = typer.Argument(None, help="Path to format (file or directory)"),
    check: bool = typer.Option(False, "--check", help="Don't write files, just check"),
    diff: bool = typer.Option(False, "--diff", help="Show diff without writing"),
    color: bool = typer.Option(False, "--color", help="Force color output"),
    no_color: bool = typer.Option(False, "--no-color", help="Disable color output"),
    line_length: str = typer.Option(None, "--line-length", help="Line length (default: 88)"),
    target_version: str = typer.Option(
        None, "--target-version", help="Python version target (e.g. py313)"
    ),
    skip_string_normalization: bool = typer.Option(
        False, "--skip-string-normalization", help="Skip string normalization"
    ),
    quiet: bool = typer.Option(False, "-q", "--quiet", help="Quiet mode"),
    verbose: bool = typer.Option(False, "-v", "--verbose", help="Verbose mode"),
    fast: bool = typer.Option(False, "--fast", help="Fast mode (skip AST safety checks)"),
    exclude: str = typer.Option(None, "--exclude", help="Exclude pattern"),
    include: str = typer.Option(None, "--include", help="Include pattern"),
):
    """Format Python code with Black"""
    if ctx.invoked_subcommand is not None:
        return

    # Build commande black
    black_builder = BlackCommandBuilder()
    cmd = black_builder.build(
        path=path,
        check=check,
        diff=diff,
        color=color,
        no_color=no_color,
        line_length=line_length,
        target_version=target_version,
        skip_string_normalization=skip_string_normalization,
        quiet=quiet,
        verbose=verbose,
        fast=fast,
        exclude=exclude,
        include=include,
    )

    # Exécution
    subprocess.run(cmd, check=True)



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.docker.format.docker_uv_command import docker_uv_command

runner = CliRunner()


class TestDockerUvCommand:
    """Tests pour la commande docker uv"""

    def test_command_without_args(self):
        """n7 dkc uv -> uv dans container"""
        with patch("n7.commands.docker.format.docker_uv_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_uv_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_uv_command, [])

                assert result.exit_code == 0
                mock_run.assert_called_once()
                cmd = mock_run.call_args[0][0]
                assert cmd[0:6] == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                )
                assert "api" in cmd
                assert "uv" in cmd

    def test_command_with_pip_install(self):
        """n7 dkc uv pip install requests"""
        with patch("n7.commands.docker.format.docker_uv_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_uv_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_uv_command, ["pip", "install", "requests"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "uv" in cmd
                assert "pip" in cmd
                assert "install" in cmd
                assert "requests" in cmd

    def test_command_with_sync(self):
        """n7 dkc uv sync"""
        with patch("n7.commands.docker.format.docker_uv_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_uv_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_uv_command, ["sync"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "uv" in cmd
                assert "sync" in cmd

    def test_command_with_lock(self):
        """n7 dkc uv lock"""
        with patch("n7.commands.docker.format.docker_uv_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_uv_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_uv_command, ["lock"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "uv" in cmd
                assert "lock" in cmd

    def test_command_with_options(self):
        """n7 dkc uv pip install requests --upgrade"""
        with patch("n7.commands.docker.format.docker_uv_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_uv_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(
                    docker_uv_command, ["pip", "install", "requests", "--upgrade"]
                )

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "uv" in cmd
                assert "pip" in cmd
                assert "install" in cmd
                assert "requests" in cmd
                assert "--upgrade" in cmd

    def test_command_with_custom_service(self):
        """n7 dkc uv --service backend sync"""
        with patch("n7.commands.docker.format.docker_uv_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_uv_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_uv_command, ["--service", "backend", "sync"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "backend" in cmd
                assert "uv" in cmd
                assert "sync" in cmd

    def test_command_with_venv(self):
        """n7 dkc uv venv"""
        with patch("n7.commands.docker.format.docker_uv_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_uv_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_uv_command, ["venv"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "uv" in cmd
                assert "venv" in cmd



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.docker.container.connect_postgres_command import connect_postgres_command

runner = CliRunner()


class TestConnectPostgresCommand:
    """Test pour la commande qui permet de se connecter a la base de donnees PostgreSQL"""

    def test_connect_postgres_command_default(self):
        """Test connexion par defaut au service 'db'"""
        with patch(
            "n7.commands.docker.container.connect_postgres_command.subprocess.run"
        ) as mock_run:
            with patch(
                "n7.commands.docker.container.connect_postgres_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(connect_postgres_command, [])

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "db",
                    "psql",
                )

    def test_connect_postgres_command_with_service(self):
        """Test connexion avec un service specifique"""
        with patch(
            "n7.commands.docker.container.connect_postgres_command.subprocess.run"
        ) as mock_run:
            with patch(
                "n7.commands.docker.container.connect_postgres_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(connect_postgres_command, ["--service", "postgres"])

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "postgres",
                    "psql",
                )

    def test_connect_postgres_command_with_user(self):
        """Test connexion avec un utilisateur specifique"""
        with patch(
            "n7.commands.docker.container.connect_postgres_command.subprocess.run"
        ) as mock_run:
            with patch(
                "n7.commands.docker.container.connect_postgres_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(connect_postgres_command, ["--user", "admin"])

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "db",
                    "psql",
                    "-U",
                    "admin",
                )

    def test_connect_postgres_command_with_database(self):
        """Test connexion avec une base de donnees specifique"""
        with patch(
            "n7.commands.docker.container.connect_postgres_command.subprocess.run"
        ) as mock_run:
            with patch(
                "n7.commands.docker.container.connect_postgres_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(connect_postgres_command, ["--database", "myapp"])

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "db",
                    "psql",
                    "-d",
                    "myapp",
                )

    def test_connect_postgres_command_with_all_options(self):
        """Test connexion avec toutes les options"""
        with patch(
            "n7.commands.docker.container.connect_postgres_command.subprocess.run"
        ) as mock_run:
            with patch(
                "n7.commands.docker.container.connect_postgres_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(
                    connect_postgres_command,
                    ["--service", "postgres", "--user", "admin", "--database", "myapp"],
                )

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "postgres",
                    "psql",
                    "-U",
                    "admin",
                    "-d",
                    "myapp",
                )

    def test_connect_postgres_command_with_short_options(self):
        """Test connexion avec les options courtes"""
        with patch(
            "n7.commands.docker.container.connect_postgres_command.subprocess.run"
        ) as mock_run:
            with patch(
                "n7.commands.docker.container.connect_postgres_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(
                    connect_postgres_command, ["-s", "postgres", "-u", "admin", "-d", "myapp"]
                )

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "postgres",
                    "psql",
                    "-U",
                    "admin",
                    "-d",
                    "myapp",
                )



from n7.commands.builders.black_command_builder import BlackCommandBuilder
from n7.commands.builders.docker_command_builder import DockerCommandBuilder
from n7.commands.builders.docker_compose_command_builder import DockerComposeCommandBuilder
from n7.commands.builders.mypy_command_builder import MypyCommandBuilder
from n7.commands.builders.pytest_command_builder import PytestCommandBuilder
from n7.commands.builders.ruff_command_builder import RuffCommandBuilder

__all__ = [
    "PytestCommandBuilder",
    "DockerCommandBuilder",
    "DockerComposeCommandBuilder",
    "BlackCommandBuilder",
    "RuffCommandBuilder",
    "MypyCommandBuilder",
]



from pathlib import Path
from typing import Any

import yaml


class ConfigN7Resolver:
    def __init__(self, base_path: Path | None = None):
        self.base_path = base_path or Path.cwd()
        self.config_file = "n7.yaml"

    def load(self) -> dict[Any, Any] | None:
        """Charge le fichier n7.yaml"""
        config_path = self.base_path / self.config_file

        if not config_path.exists():
            return None

        with open(config_path) as f:
            result: Any = yaml.safe_load(f)
            return result if isinstance(result, dict) else None

    def get_docker_config(self) -> dict[Any, Any] | None:
        """Retourne la section docker de la config"""
        config = self.load()

        if config is None:
            return None

        result = config.get("docker")
        return result if isinstance(result, dict) else None

    def get_path_env_file(self) -> str | None:
        """Retourne path_env_file de la config docker"""
        docker_config = self.get_docker_config()

        if docker_config is None:
            return None

        return docker_config.get("path_env_file")

    def get_path_compose_file(self) -> str | None:
        """Retourne path_compose_file de la config docker"""
        docker_config = self.get_docker_config()

        if docker_config is None:
            return None

        return docker_config.get("path_compose_file")

    def get_default_service(self) -> str | None:
        """Retourne default_service de la config docker"""
        docker_config = self.get_docker_config()

        if docker_config is None:
            return None

        return docker_config.get("default_service")



import subprocess
from typing import List

import typer

local_uv_command = typer.Typer(
    help="Run uv command with any arguments",
    invoke_without_command=True,
    no_args_is_help=False,
)


@local_uv_command.callback()
def run(
    args: List[str] = typer.Argument(None, help="Arguments to pass to uv command"),
):
    """Run uv command with any arguments"""
    # Build command uv
    uv_args = args if args else []
    cmd = ("uv", *uv_args)

    # Exécution
    subprocess.run(cmd, check=True)



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.local.format.local_ruff_command import local_ruff_command

runner = CliRunner()


class TestLocalRuffCommand:
    """Tests pour la commande ruff local"""

    def test_command_without_options(self):
        """command n7 pyrf -> ruff check simple"""
        with patch("n7.commands.local.format.local_ruff_command.subprocess.run") as mock_run:
            result = runner.invoke(local_ruff_command, [])

            assert result.exit_code == 0
            mock_run.assert_called_once()
            cmd = mock_run.call_args[0][0]
            assert cmd == ("ruff", "check")

    def test_command_with_path(self):
        """command n7 pyrf src/"""
        with patch("n7.commands.local.format.local_ruff_command.subprocess.run") as mock_run:
            result = runner.invoke(local_ruff_command, ["src/"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert cmd == ("ruff", "check", "src/")

    def test_command_with_fix(self):
        """command n7 pyrf --fix"""
        with patch("n7.commands.local.format.local_ruff_command.subprocess.run") as mock_run:
            result = runner.invoke(local_ruff_command, ["--fix"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--fix" in cmd

    def test_command_with_diff(self):
        """command n7 pyrf --diff"""
        with patch("n7.commands.local.format.local_ruff_command.subprocess.run") as mock_run:
            result = runner.invoke(local_ruff_command, ["--diff"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--diff" in cmd

    def test_command_with_select(self):
        """command n7 pyrf --select E,F"""
        with patch("n7.commands.local.format.local_ruff_command.subprocess.run") as mock_run:
            result = runner.invoke(local_ruff_command, ["--select", "E,F"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--select" in cmd
            assert "E,F" in cmd

    def test_command_with_ignore(self):
        """command n7 pyrf --ignore E501"""
        with patch("n7.commands.local.format.local_ruff_command.subprocess.run") as mock_run:
            result = runner.invoke(local_ruff_command, ["--ignore", "E501"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--ignore" in cmd
            assert "E501" in cmd

    def test_command_with_verbose(self):
        """command n7 pyrf -v"""
        with patch("n7.commands.local.format.local_ruff_command.subprocess.run") as mock_run:
            result = runner.invoke(local_ruff_command, ["-v"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--verbose" in cmd

    def test_command_combined_options(self):
        """command n7 pyrf src/ --fix --diff -v"""
        with patch("n7.commands.local.format.local_ruff_command.subprocess.run") as mock_run:
            result = runner.invoke(
                local_ruff_command,
                [
                    "--fix",
                    "--diff",
                    "-v",
                    "src/",
                ],
            )

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--fix" in cmd
            assert "--diff" in cmd
            assert "--verbose" in cmd
            assert cmd[-1] == "src/"

    def test_command_with_rules_options(self):
        """command n7 pyrf with rules options"""
        with patch("n7.commands.local.format.local_ruff_command.subprocess.run") as mock_run:
            result = runner.invoke(
                local_ruff_command,
                [
                    "--select",
                    "E,F",
                    "--ignore",
                    "E501",
                    "--extend-select",
                    "B",
                ],
            )

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--select" in cmd
            assert "E,F" in cmd
            assert "--ignore" in cmd
            assert "E501" in cmd
            assert "--extend-select" in cmd
            assert "B" in cmd



from n7.commands.builders import MypyCommandBuilder


class TestMypyCommandBuilder:
    def setup_method(self):
        self.builder = MypyCommandBuilder()

    def test_instance_builder(self):
        """test instance builder"""
        assert self.builder is not None
        assert self.builder.base_cmd == "mypy"

    def test_build_command_without_all(self):
        """command mypy only"""
        res = self.builder.build()
        assert res == ("mypy",)

    def test_build_command_with_path(self):
        """command mypy with path"""
        res = self.builder.build(path="my/path")
        assert res == ("mypy", "my/path")

    def test_build_with_strict(self):
        """command mypy --strict"""
        result = self.builder.build(strict=True)
        assert result == ("mypy", "--strict")

    def test_build_with_check_untyped_defs(self):
        """command mypy --check-untyped-defs"""
        result = self.builder.build(check_untyped_defs=True)
        assert result == ("mypy", "--check-untyped-defs")

    def test_build_with_disallow_untyped_defs(self):
        """command mypy --disallow-untyped-defs"""
        result = self.builder.build(disallow_untyped_defs=True)
        assert result == ("mypy", "--disallow-untyped-defs")

    def test_build_with_disallow_any_expr(self):
        """command mypy --disallow-any-expr"""
        result = self.builder.build(disallow_any_expr=True)
        assert result == ("mypy", "--disallow-any-expr")

    def test_build_with_no_implicit_optional(self):
        """command mypy --no-implicit-optional"""
        result = self.builder.build(no_implicit_optional=True)
        assert result == ("mypy", "--no-implicit-optional")

    def test_build_with_warn_return_any(self):
        """command mypy --warn-return-any"""
        result = self.builder.build(warn_return_any=True)
        assert result == ("mypy", "--warn-return-any")

    def test_build_with_warn_unused_ignores(self):
        """command mypy --warn-unused-ignores"""
        result = self.builder.build(warn_unused_ignores=True)
        assert result == ("mypy", "--warn-unused-ignores")

    def test_build_with_show_error_codes(self):
        """command mypy --show-error-codes"""
        result = self.builder.build(show_error_codes=True)
        assert result == ("mypy", "--show-error-codes")

    def test_build_with_pretty(self):
        """command mypy --pretty"""
        result = self.builder.build(pretty=True)
        assert result == ("mypy", "--pretty")

    def test_build_with_no_error_summary(self):
        """command mypy --no-error-summary"""
        result = self.builder.build(no_error_summary=True)
        assert result == ("mypy", "--no-error-summary")

    def test_build_with_config_file(self):
        """command mypy --config-file pyproject.toml"""
        result = self.builder.build(config_file="pyproject.toml")
        assert result == ("mypy", "--config-file", "pyproject.toml")

    def test_build_with_python_version(self):
        """command mypy --python-version 3.13"""
        result = self.builder.build(python_version="3.13")
        assert result == ("mypy", "--python-version", "3.13")

    def test_build_with_exclude(self):
        """command mypy --exclude venv"""
        result = self.builder.build(exclude="venv")
        assert result == ("mypy", "--exclude", "venv")

    def test_build_with_verbose(self):
        """command mypy --verbose"""
        result = self.builder.build(verbose=True)
        assert result == ("mypy", "--verbose")

    def test_build_with_quiet(self):
        """command mypy --quiet"""
        result = self.builder.build(quiet=True)
        assert result == ("mypy", "--quiet")

    def test_build_with_multiple_options(self):
        """command mypy with many options"""
        result = self.builder.build(
            path="src/",
            strict=True,
            show_error_codes=True,
            verbose=True,
        )
        assert result == ("mypy", "--strict", "--show-error-codes", "--verbose", "src/")

    def test_build_with_type_checking_options(self):
        """command mypy with type checking options"""
        result = self.builder.build(
            check_untyped_defs=True,
            disallow_untyped_defs=True,
            warn_return_any=True,
        )
        assert result == (
            "mypy",
            "--check-untyped-defs",
            "--disallow-untyped-defs",
            "--warn-return-any",
        )

    def test_path_always_last(self):
        """This path always last"""
        result = self.builder.build(
            path="src/",
            strict=True,
            python_version="3.13",
            verbose=True,
        )
        assert result[-1] == "src/"

    def test_build_full_command(self):
        """command mypy with all common options"""
        result = self.builder.build(
            path=".",
            strict=True,
            show_error_codes=True,
            pretty=True,
            verbose=True,
        )
        assert result == (
            "mypy",
            "--strict",
            "--show-error-codes",
            "--pretty",
            "--verbose",
            ".",
        )



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.local.format.local_mypy_command import local_mypy_command

runner = CliRunner()


class TestLocalMypyCommand:
    """Tests pour la commande mypy local"""

    def test_command_without_options(self):
        """command n7 pymy -> mypy simple"""
        with patch("n7.commands.local.format.local_mypy_command.subprocess.run") as mock_run:
            result = runner.invoke(local_mypy_command, [])

            assert result.exit_code == 0
            mock_run.assert_called_once()
            cmd = mock_run.call_args[0][0]
            assert cmd == ("mypy",)

    def test_command_with_path(self):
        """command n7 pymy src/"""
        with patch("n7.commands.local.format.local_mypy_command.subprocess.run") as mock_run:
            result = runner.invoke(local_mypy_command, ["src/"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert cmd == ("mypy", "src/")

    def test_command_with_strict(self):
        """command n7 pymy --strict"""
        with patch("n7.commands.local.format.local_mypy_command.subprocess.run") as mock_run:
            result = runner.invoke(local_mypy_command, ["--strict"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--strict" in cmd

    def test_command_with_show_error_codes(self):
        """command n7 pymy --show-error-codes"""
        with patch("n7.commands.local.format.local_mypy_command.subprocess.run") as mock_run:
            result = runner.invoke(local_mypy_command, ["--show-error-codes"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--show-error-codes" in cmd

    def test_command_with_python_version(self):
        """command n7 pymy --python-version 3.13"""
        with patch("n7.commands.local.format.local_mypy_command.subprocess.run") as mock_run:
            result = runner.invoke(local_mypy_command, ["--python-version", "3.13"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--python-version" in cmd
            assert "3.13" in cmd

    def test_command_with_config_file(self):
        """command n7 pymy --config-file pyproject.toml"""
        with patch("n7.commands.local.format.local_mypy_command.subprocess.run") as mock_run:
            result = runner.invoke(local_mypy_command, ["--config-file", "pyproject.toml"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--config-file" in cmd
            assert "pyproject.toml" in cmd

    def test_command_with_verbose(self):
        """command n7 pymy -v"""
        with patch("n7.commands.local.format.local_mypy_command.subprocess.run") as mock_run:
            result = runner.invoke(local_mypy_command, ["-v"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--verbose" in cmd

    def test_command_combined_options(self):
        """command n7 pymy src/ --strict --show-error-codes -v"""
        with patch("n7.commands.local.format.local_mypy_command.subprocess.run") as mock_run:
            result = runner.invoke(
                local_mypy_command,
                [
                    "--strict",
                    "--show-error-codes",
                    "-v",
                    "src/",
                ],
            )

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--strict" in cmd
            assert "--show-error-codes" in cmd
            assert "--verbose" in cmd
            assert cmd[-1] == "src/"

    def test_command_with_type_checking_options(self):
        """command n7 pymy with type checking options"""
        with patch("n7.commands.local.format.local_mypy_command.subprocess.run") as mock_run:
            result = runner.invoke(
                local_mypy_command,
                [
                    "--check-untyped-defs",
                    "--disallow-untyped-defs",
                    "--warn-return-any",
                ],
            )

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--check-untyped-defs" in cmd
            assert "--disallow-untyped-defs" in cmd
            assert "--warn-return-any" in cmd



import subprocess

import typer

from n7.commands.builders import DockerComposeCommandBuilder, MypyCommandBuilder
from n7.commands.resolver import DockerFileResolver

docker_mypy_command = typer.Typer(help="Type check Python code with Mypy in Docker container")


@docker_mypy_command.callback(invoke_without_command=True)
def run(
    ctx: typer.Context,
    path: str = typer.Argument(None, help="Path to type check (file or directory)"),
    strict: bool = typer.Option(False, "--strict", help="Strict mode"),
    check_untyped_defs: bool = typer.Option(
        False, "--check-untyped-defs", help="Check untyped functions"
    ),
    disallow_untyped_defs: bool = typer.Option(
        False, "--disallow-untyped-defs", help="Disallow untyped functions"
    ),
    disallow_any_expr: bool = typer.Option(False, "--disallow-any-expr", help="Disallow Any"),
    no_implicit_optional: bool = typer.Option(
        False, "--no-implicit-optional", help="No implicit optional"
    ),
    warn_return_any: bool = typer.Option(False, "--warn-return-any", help="Warn on return Any"),
    warn_unused_ignores: bool = typer.Option(
        False, "--warn-unused-ignores", help="Warn on unused ignores"
    ),
    show_error_codes: bool = typer.Option(False, "--show-error-codes", help="Show error codes"),
    pretty: bool = typer.Option(False, "--pretty", help="Pretty output"),
    no_error_summary: bool = typer.Option(False, "--no-error-summary", help="No error summary"),
    config_file: str = typer.Option(None, "--config-file", help="Config file path"),
    python_version: str = typer.Option(None, "--python-version", help="Python version (e.g. 3.13)"),
    exclude: str = typer.Option(None, "--exclude", help="Exclude pattern"),
    verbose: bool = typer.Option(False, "-v", "--verbose", help="Verbose mode"),
    quiet: bool = typer.Option(False, "-q", "--quiet", help="Quiet mode"),
    service: str = typer.Option(None, "--service", "-s", help="Service docker"),
):
    """Type check Python code with Mypy in Docker container"""
    if ctx.invoked_subcommand is not None:
        return

    # Résolution des fichiers docker
    resolver = DockerFileResolver()
    config_docker = resolver.resolve()

    # Service à utiliser
    target_service = service or config_docker["default_service"] or "api"

    # Build commande mypy
    mypy_builder = MypyCommandBuilder()
    mypy_cmd = mypy_builder.build(
        path=path,
        strict=strict,
        check_untyped_defs=check_untyped_defs,
        disallow_untyped_defs=disallow_untyped_defs,
        disallow_any_expr=disallow_any_expr,
        no_implicit_optional=no_implicit_optional,
        warn_return_any=warn_return_any,
        warn_unused_ignores=warn_unused_ignores,
        show_error_codes=show_error_codes,
        pretty=pretty,
        no_error_summary=no_error_summary,
        config_file=config_file,
        python_version=python_version,
        exclude=exclude,
        verbose=verbose,
        quiet=quiet,
    )

    # Build commande docker compose
    docker_builder = DockerComposeCommandBuilder(
        env_file=config_docker["env_file"] or ".env",
        compose_file=config_docker["compose_file"] or "docker-compose.yml",
    )
    cmd = docker_builder.build_exec(service=target_service, cmd=mypy_cmd)

    # Exécution
    subprocess.run(cmd, check=True)



# commands/resolver/docker_file_resolver.py

from pathlib import Path

from n7.commands.resolver.config_n7_resolver import ConfigN7Resolver

# Extensions
EXTENSIONS = ("yaml", "yml")

# Environnements
ENVS = (
    "",
    "dev",
    "development",
    "prod",
    "production",
    "staging",
    "preprod",
    "test",
    "local",
    "api",
)

# Préfixes dossiers pour compose
COMPOSE_FOLDERS = (
    "",
    "docker/",
)

# Préfixes dossiers pour .env
ENV_FOLDERS = (
    "",
    "api/",
    "app/",
    "front/",
    "back/",
    "server/",
    "frontend/",
    "backend/",
)


def _generate_compose_files() -> tuple[str, ...]:
    """Génère la liste des fichiers compose à chercher"""
    files = []
    for folder in COMPOSE_FOLDERS:
        for ext in EXTENSIONS:
            for env in ENVS:
                suffix = f".{env}" if env else ""
                files.append(f"{folder}docker-compose{suffix}.{ext}")
                files.append(f"{folder}compose{suffix}.{ext}")
    return tuple(files)


def _generate_env_files() -> tuple[str, ...]:
    """Génère la liste des fichiers env à chercher"""
    files = []
    for folder in ENV_FOLDERS:
        for env in ENVS:
            suffix = f".{env}" if env else ""
            files.append(f"{folder}.env{suffix}")
    return tuple(files)


COMPOSE_FILES = _generate_compose_files()
ENV_FILES = _generate_env_files()


class DockerFileResolver:
    """Résout les fichiers docker (.env et compose)"""

    def __init__(self, base_path: Path | None = None):
        self.base_path = base_path or Path.cwd()
        self.config_resolver = ConfigN7Resolver(base_path=self.base_path)

    def find_compose_file(self) -> str | None:
        """Trouve le premier fichier compose existant"""
        for file in COMPOSE_FILES:
            if (self.base_path / file).exists():
                return file
        return None

    def find_env_file(self) -> str | None:
        """Trouve le premier fichier env existant"""
        for file in ENV_FILES:
            if (self.base_path / file).exists():
                return file
        return None

    def resolve(self) -> dict:
        """Résout les fichiers docker avec priorité config > auto"""
        config_env = self.config_resolver.get_path_env_file()
        config_compose = self.config_resolver.get_path_compose_file()
        default_service = self.config_resolver.get_default_service()

        return {
            "env_file": config_env or self.find_env_file(),
            "compose_file": config_compose or self.find_compose_file(),
            "default_service": default_service,
        }



from n7.commands.builders import PytestCommandBuilder


class TestPytestCommandBuilder:
    def setup_method(self):
        self.builder = PytestCommandBuilder()

    def test_instance_builder(self):
        """test instance builder"""
        assert self.builder is not None
        assert self.builder.base_cmd == "pytest"

    def test_build_command_without_all(self):
        """command pytest only"""
        res = self.builder.build()
        assert res == ("pytest",)

    def test_build_command_with_path(self):
        """command pytest with path"""
        res = self.builder.build(path="my/path")
        assert res == ("pytest", "my/path")

    def test_build_with_verbose(self):
        """command pytest -v"""
        result = self.builder.build(verbose=True)
        assert result == ("pytest", "-v")

    def test_build_with_stop_first(self):
        """command pytest -x"""
        result = self.builder.build(stop_first=True)
        assert result == ("pytest", "-x")

    def test_build_with_filter_tests(self):
        """command pytest -k test_name"""
        result = self.builder.build(filter_tests="test_name")
        assert result == ("pytest", "-k", "test_name")

    def test_build_with_workers(self):
        """command pytest -n auto"""
        result = self.builder.build(workers="auto")
        assert result == ("pytest", "-n", "auto")

    def test_build_with_workers_number(self):
        """command pytest -n 4"""
        result = self.builder.build(workers="4")
        assert result == ("pytest", "-n", "4")

    def test_build_with_cov(self):
        """command pytest --cov=app"""
        result = self.builder.build(cov="users")
        assert result == ("pytest", "--cov=users")

    def test_build_with_cov_report(self):
        """command pytest --cov-report=html"""
        result = self.builder.build(cov_report="html")
        assert result == ("pytest", "--cov-report=html")

    def test_build_with_last_failed(self):
        """command pytest --lf"""
        result = self.builder.build(last_failed=True)
        assert result == ("pytest", "--lf")

    def test_build_with_failed_first(self):
        """command pytest --ff"""
        result = self.builder.build(failed_first=True)
        assert result == ("pytest", "--ff")

    def test_build_with_create_db(self):
        """command pytest --create-db"""
        result = self.builder.build(create_db=True)
        assert result == ("pytest", "--create-db")

    def test_build_with_migrations(self):
        """command pytest --migrations"""
        result = self.builder.build(migrations=True)
        assert result == ("pytest", "--migrations")

    def test_build_with_multiple_options(self):
        """command pytest with many options"""
        result = self.builder.build(
            path="users/tests/",
            verbose=True,
            workers="auto",
        )
        assert result == ("pytest", "-v", "-n", "auto", "users/tests/")

    def test_build_with_coverage_full(self):
        """command pytest coverage completed"""
        result = self.builder.build(
            cov="users",
            cov_report="html",
            verbose=True,
        )
        assert result == ("pytest", "-v", "--cov=users", "--cov-report=html")

    def test_path_always_last(self):
        """This path always last"""
        result = self.builder.build(
            path="tests/",
            verbose=True,
            cov="app",
            workers="auto",
        )
        assert result[-1] == "tests/"



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.local.format.local_uv_command import local_uv_command

runner = CliRunner()


class TestLocalUvCommand:
    """Tests pour la commande uv local"""

    def test_command_without_args(self):
        """command n7 uv -> uv"""
        with patch("n7.commands.local.format.local_uv_command.subprocess.run") as mock_run:
            result = runner.invoke(local_uv_command, [])

            assert result.exit_code == 0
            mock_run.assert_called_once()
            cmd = mock_run.call_args[0][0]
            assert cmd == ("uv",)

    def test_command_with_pip_install(self):
        """command n7 uv pip install requests"""
        with patch("n7.commands.local.format.local_uv_command.subprocess.run") as mock_run:
            result = runner.invoke(local_uv_command, ["pip", "install", "requests"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert cmd == ("uv", "pip", "install", "requests")

    def test_command_with_sync(self):
        """command n7 uv sync"""
        with patch("n7.commands.local.format.local_uv_command.subprocess.run") as mock_run:
            result = runner.invoke(local_uv_command, ["sync"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert cmd == ("uv", "sync")

    def test_command_with_lock(self):
        """command n7 uv lock"""
        with patch("n7.commands.local.format.local_uv_command.subprocess.run") as mock_run:
            result = runner.invoke(local_uv_command, ["lock"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert cmd == ("uv", "lock")

    def test_command_with_options(self):
        """command n7 uv pip install requests --upgrade"""
        with patch("n7.commands.local.format.local_uv_command.subprocess.run") as mock_run:
            result = runner.invoke(local_uv_command, ["pip", "install", "requests", "--upgrade"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert cmd == ("uv", "pip", "install", "requests", "--upgrade")

    def test_command_with_multiple_packages(self):
        """command n7 uv pip install requests pytest black"""
        with patch("n7.commands.local.format.local_uv_command.subprocess.run") as mock_run:
            result = runner.invoke(
                local_uv_command, ["pip", "install", "requests", "pytest", "black"]
            )

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert cmd == ("uv", "pip", "install", "requests", "pytest", "black")

    def test_command_with_venv(self):
        """command n7 uv venv"""
        with patch("n7.commands.local.format.local_uv_command.subprocess.run") as mock_run:
            result = runner.invoke(local_uv_command, ["venv"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert cmd == ("uv", "venv")



import subprocess
from typing import Annotated

import typer

from n7.commands.builders import DockerComposeCommandBuilder
from n7.commands.resolver import DockerFileResolver

py_manage_command = typer.Typer(help="Run python manage.py in container")


@py_manage_command.callback(invoke_without_command=True)
def manage(
    args: Annotated[list[str] | None, typer.Argument(help="Arguments for manage.py")] = None,
    service: Annotated[str | None, typer.Option("--service", "-s", help="Service docker")] = None,
):
    resolver = DockerFileResolver()
    config = resolver.resolve()

    target_service = service or config["default_service"] or "api"

    # Build commande docker compose
    docker_builder = DockerComposeCommandBuilder(
        env_file=config["env_file"] or ".env",
        compose_file=config["compose_file"] or "docker-compose.yml",
    )

    # Construire la commande manage.py
    command = ["python", "manage.py"]
    if args:
        command.extend(args)

    cmd = docker_builder.build_exec(service=target_service, cmd=tuple(command), disable_tty=False)

    subprocess.run(cmd, check=True)



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.docker.container.connect_shell_command import connect_shell_command

runner = CliRunner()


class TestConnectShellCommand:
    """Test pour la commande qui permet de se connecter au shell du container"""

    def test_connect_shell_command(self):
        with patch("n7.commands.docker.container.connect_shell_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.connect_shell_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(connect_shell_command, [])

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "api",
                    "bash",
                )

    def test_connect_shell_command_with_no_bash(self):
        with patch("n7.commands.docker.container.connect_shell_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.connect_shell_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(connect_shell_command, ["--no-bash"])

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "api",
                    "sh",
                )

    def test_connect_shell_command_with_service(self):
        with patch("n7.commands.docker.container.connect_shell_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.connect_shell_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(connect_shell_command, ["--service", "truc"])

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "truc",
                    "bash",
                )

    def test_connect_shell_command_with_service_with_no_bash(self):
        with patch("n7.commands.docker.container.connect_shell_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.connect_shell_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": None,
                }

                result = runner.invoke(connect_shell_command, ["--no-bash", "--service", "truc"])

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "truc",
                    "sh",
                )



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.local.format.local_black_command import local_black_command

runner = CliRunner()


class TestLocalBlackCommand:
    """Tests pour la commande black local"""

    def test_command_without_options(self):
        """command n7 pybl -> black simple"""
        with patch("n7.commands.local.format.local_black_command.subprocess.run") as mock_run:
            result = runner.invoke(local_black_command, [])

            assert result.exit_code == 0
            mock_run.assert_called_once()
            cmd = mock_run.call_args[0][0]
            assert cmd == ("black",)

    def test_command_with_path(self):
        """command n7 pybl src/"""
        with patch("n7.commands.local.format.local_black_command.subprocess.run") as mock_run:
            result = runner.invoke(local_black_command, ["src/"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert cmd == ("black", "src/")

    def test_command_with_check(self):
        """command n7 pybl --check"""
        with patch("n7.commands.local.format.local_black_command.subprocess.run") as mock_run:
            result = runner.invoke(local_black_command, ["--check"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--check" in cmd

    def test_command_with_diff(self):
        """command n7 pybl --diff"""
        with patch("n7.commands.local.format.local_black_command.subprocess.run") as mock_run:
            result = runner.invoke(local_black_command, ["--diff"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--diff" in cmd

    def test_command_with_line_length(self):
        """command n7 pybl --line-length 100"""
        with patch("n7.commands.local.format.local_black_command.subprocess.run") as mock_run:
            result = runner.invoke(local_black_command, ["--line-length", "100"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--line-length" in cmd
            assert "100" in cmd

    def test_command_with_target_version(self):
        """command n7 pybl --target-version py313"""
        with patch("n7.commands.local.format.local_black_command.subprocess.run") as mock_run:
            result = runner.invoke(local_black_command, ["--target-version", "py313"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--target-version" in cmd
            assert "py313" in cmd

    def test_command_with_verbose(self):
        """command n7 pybl -v"""
        with patch("n7.commands.local.format.local_black_command.subprocess.run") as mock_run:
            result = runner.invoke(local_black_command, ["-v"])

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--verbose" in cmd

    def test_command_combined_options(self):
        """command n7 pybl src/ --check --diff -v"""
        with patch("n7.commands.local.format.local_black_command.subprocess.run") as mock_run:
            result = runner.invoke(
                local_black_command,
                [
                    "--check",
                    "--diff",
                    "-v",
                    "src/",
                ],
            )

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--check" in cmd
            assert "--diff" in cmd
            assert "--verbose" in cmd
            assert cmd[-1] == "src/"

    def test_command_with_formatting_options(self):
        """command n7 pybl with formatting options"""
        with patch("n7.commands.local.format.local_black_command.subprocess.run") as mock_run:
            result = runner.invoke(
                local_black_command,
                [
                    "--line-length",
                    "100",
                    "--target-version",
                    "py313",
                    "--skip-string-normalization",
                ],
            )

            assert result.exit_code == 0
            cmd = mock_run.call_args[0][0]
            assert "--line-length" in cmd
            assert "100" in cmd
            assert "--target-version" in cmd
            assert "py313" in cmd
            assert "--skip-string-normalization" in cmd



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.local.format.local_pycheck_command import local_pycheck_command

runner = CliRunner()


class TestLocalPycheckCommand:
    """Tests pour la commande pycheck local"""

    def test_command_without_options(self):
        """command n7 pycheck -> pytest, black --check, ruff check, mypy"""
        with patch("n7.commands.local.format.local_pycheck_command.subprocess.run") as mock_run:
            result = runner.invoke(local_pycheck_command, [])

            assert result.exit_code == 0
            assert mock_run.call_count == 4

            # Vérifier les commandes appelées
            calls = mock_run.call_args_list
            pytest_cmd = calls[0][0][0]
            black_cmd = calls[1][0][0]
            ruff_cmd = calls[2][0][0]
            mypy_cmd = calls[3][0][0]

            assert pytest_cmd == ("pytest",)
            assert black_cmd == ("black", "--check", ".")
            assert ruff_cmd == ("ruff", "check", ".")
            assert mypy_cmd == ("mypy", ".")

    def test_command_with_path(self):
        """command n7 pycheck src/"""
        with patch("n7.commands.local.format.local_pycheck_command.subprocess.run") as mock_run:
            result = runner.invoke(local_pycheck_command, ["src/"])

            assert result.exit_code == 0
            assert mock_run.call_count == 4

            calls = mock_run.call_args_list
            pytest_cmd = calls[0][0][0]
            black_cmd = calls[1][0][0]
            ruff_cmd = calls[2][0][0]
            mypy_cmd = calls[3][0][0]

            assert pytest_cmd == ("pytest", "src/")
            assert black_cmd == ("black", "--check", "src/")
            assert ruff_cmd == ("ruff", "check", "src/")
            assert mypy_cmd == ("mypy", "src/")

    def test_command_with_fix(self):
        """command n7 pycheck --fix -> pytest, black (format), ruff check --fix, mypy"""
        with patch("n7.commands.local.format.local_pycheck_command.subprocess.run") as mock_run:
            result = runner.invoke(local_pycheck_command, ["--fix"])

            assert result.exit_code == 0
            assert mock_run.call_count == 4

            calls = mock_run.call_args_list
            pytest_cmd = calls[0][0][0]
            black_cmd = calls[1][0][0]
            ruff_cmd = calls[2][0][0]
            mypy_cmd = calls[3][0][0]

            # Pytest reste inchangé
            assert pytest_cmd == ("pytest",)
            # Black sans --check (mode format)
            assert black_cmd == ("black", ".")
            # Ruff avec --fix
            assert ruff_cmd == ("ruff", "check", "--fix", ".")
            # Mypy reste inchangé
            assert mypy_cmd == ("mypy", ".")

    def test_command_with_fix_and_path(self):
        """command n7 pycheck --fix src/"""
        with patch("n7.commands.local.format.local_pycheck_command.subprocess.run") as mock_run:
            result = runner.invoke(local_pycheck_command, ["--fix", "src/"])

            assert result.exit_code == 0
            assert mock_run.call_count == 4

            calls = mock_run.call_args_list
            pytest_cmd = calls[0][0][0]
            black_cmd = calls[1][0][0]
            ruff_cmd = calls[2][0][0]
            mypy_cmd = calls[3][0][0]

            assert pytest_cmd == ("pytest", "src/")
            assert black_cmd == ("black", "src/")
            assert ruff_cmd == ("ruff", "check", "--fix", "src/")
            assert mypy_cmd == ("mypy", "src/")

    def test_commands_executed_sequentially(self):
        """Vérifie que les commandes sont exécutées dans l'ordre: pytest, black, ruff, mypy"""
        with patch("n7.commands.local.format.local_pycheck_command.subprocess.run") as mock_run:
            runner.invoke(local_pycheck_command, [])

            calls = mock_run.call_args_list
            assert len(calls) == 4
            assert calls[0][0][0][0] == "pytest"
            assert calls[1][0][0][0] == "black"
            assert calls[2][0][0][0] == "ruff"
            assert calls[3][0][0][0] == "mypy"

    def test_command_with_verbose(self):
        """command n7 pycheck -v -> pytest -v"""
        with patch("n7.commands.local.format.local_pycheck_command.subprocess.run") as mock_run:
            result = runner.invoke(local_pycheck_command, ["-v"])

            assert result.exit_code == 0
            calls = mock_run.call_args_list
            pytest_cmd = calls[0][0][0]
            assert pytest_cmd == ("pytest", "-v")

    def test_command_with_stop_first(self):
        """command n7 pycheck -x -> pytest -x"""
        with patch("n7.commands.local.format.local_pycheck_command.subprocess.run") as mock_run:
            result = runner.invoke(local_pycheck_command, ["-x"])

            assert result.exit_code == 0
            calls = mock_run.call_args_list
            pytest_cmd = calls[0][0][0]
            assert pytest_cmd == ("pytest", "-x")

    def test_command_with_filter_tests(self):
        """command n7 pycheck -k test_name -> pytest -k test_name"""
        with patch("n7.commands.local.format.local_pycheck_command.subprocess.run") as mock_run:
            result = runner.invoke(local_pycheck_command, ["-k", "test_name"])

            assert result.exit_code == 0
            calls = mock_run.call_args_list
            pytest_cmd = calls[0][0][0]
            assert pytest_cmd == ("pytest", "-k", "test_name")

    def test_command_with_all_pytest_options(self):
        """command n7 pycheck -v -x -k test_name src/"""
        with patch("n7.commands.local.format.local_pycheck_command.subprocess.run") as mock_run:
            result = runner.invoke(local_pycheck_command, ["-v", "-x", "-k", "test_name", "src/"])

            assert result.exit_code == 0
            calls = mock_run.call_args_list
            pytest_cmd = calls[0][0][0]
            assert "-v" in pytest_cmd
            assert "-x" in pytest_cmd
            assert "-k" in pytest_cmd
            assert "test_name" in pytest_cmd
            assert pytest_cmd[-1] == "src/"



from n7.commands.builders import RuffCommandBuilder


class TestRuffCommandBuilder:
    def setup_method(self):
        self.builder = RuffCommandBuilder()

    def test_instance_builder(self):
        """test instance builder"""
        assert self.builder is not None
        assert self.builder.base_cmd == "ruff"

    def test_build_command_without_all(self):
        """command ruff check only"""
        res = self.builder.build()
        assert res == ("ruff", "check")

    def test_build_command_with_path(self):
        """command ruff check with path"""
        res = self.builder.build(path="my/path")
        assert res == ("ruff", "check", "my/path")

    def test_build_with_fix(self):
        """command ruff check --fix"""
        result = self.builder.build(fix=True)
        assert result == ("ruff", "check", "--fix")

    def test_build_with_diff(self):
        """command ruff check --diff"""
        result = self.builder.build(diff=True)
        assert result == ("ruff", "check", "--diff")

    def test_build_with_watch(self):
        """command ruff check --watch"""
        result = self.builder.build(watch=True)
        assert result == ("ruff", "check", "--watch")

    def test_build_with_fix_only(self):
        """command ruff check --fix-only"""
        result = self.builder.build(fix_only=True)
        assert result == ("ruff", "check", "--fix-only")

    def test_build_with_unsafe_fixes(self):
        """command ruff check --unsafe-fixes"""
        result = self.builder.build(unsafe_fixes=True)
        assert result == ("ruff", "check", "--unsafe-fixes")

    def test_build_with_show_fixes(self):
        """command ruff check --show-fixes"""
        result = self.builder.build(show_fixes=True)
        assert result == ("ruff", "check", "--show-fixes")

    def test_build_with_output_format(self):
        """command ruff check --output-format json"""
        result = self.builder.build(output_format="json")
        assert result == ("ruff", "check", "--output-format", "json")

    def test_build_with_config(self):
        """command ruff check --config pyproject.toml"""
        result = self.builder.build(config="pyproject.toml")
        assert result == ("ruff", "check", "--config", "pyproject.toml")

    def test_build_with_select(self):
        """command ruff check --select E,F"""
        result = self.builder.build(select="E,F")
        assert result == ("ruff", "check", "--select", "E,F")

    def test_build_with_ignore(self):
        """command ruff check --ignore E501"""
        result = self.builder.build(ignore="E501")
        assert result == ("ruff", "check", "--ignore", "E501")

    def test_build_with_exclude(self):
        """command ruff check --exclude venv"""
        result = self.builder.build(exclude="venv")
        assert result == ("ruff", "check", "--exclude", "venv")

    def test_build_with_extend_select(self):
        """command ruff check --extend-select B"""
        result = self.builder.build(extend_select="B")
        assert result == ("ruff", "check", "--extend-select", "B")

    def test_build_with_quiet(self):
        """command ruff check --quiet"""
        result = self.builder.build(quiet=True)
        assert result == ("ruff", "check", "--quiet")

    def test_build_with_verbose(self):
        """command ruff check --verbose"""
        result = self.builder.build(verbose=True)
        assert result == ("ruff", "check", "--verbose")

    def test_build_with_multiple_options(self):
        """command ruff check with many options"""
        result = self.builder.build(
            path="src/",
            fix=True,
            diff=True,
            verbose=True,
        )
        assert result == ("ruff", "check", "--fix", "--diff", "--verbose", "src/")

    def test_build_with_rules_options(self):
        """command ruff check with rules options"""
        result = self.builder.build(
            select="E,F",
            ignore="E501",
            extend_select="B",
        )
        assert result == (
            "ruff",
            "check",
            "--select",
            "E,F",
            "--ignore",
            "E501",
            "--extend-select",
            "B",
        )

    def test_path_always_last(self):
        """This path always last"""
        result = self.builder.build(
            path="src/",
            fix=True,
            select="E,F",
            verbose=True,
        )
        assert result[-1] == "src/"

    def test_build_full_command(self):
        """command ruff check with all common options"""
        result = self.builder.build(
            path=".",
            fix=True,
            diff=True,
            select="E,F",
            verbose=True,
        )
        assert result == (
            "ruff",
            "check",
            "--fix",
            "--diff",
            "--select",
            "E,F",
            "--verbose",
            ".",
        )



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.docker.format.docker_pyqa_command import docker_pyqa_command

runner = CliRunner()


class TestDockerPyqaCommand:
    """Tests pour la commande docker pyqa"""

    def test_command_without_options(self):
        """n7 dkc pyqa -> black --check, ruff check, mypy dans container"""
        with patch("n7.commands.docker.format.docker_pyqa_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_pyqa_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_pyqa_command, [])

                assert result.exit_code == 0
                assert mock_run.call_count == 3

                # Vérifier les commandes appelées
                calls = mock_run.call_args_list

                # Black command
                black_cmd = calls[0][0][0]
                assert black_cmd[0:6] == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                )
                assert "api" in black_cmd
                assert "black" in black_cmd
                assert "--check" in black_cmd

                # Ruff command
                ruff_cmd = calls[1][0][0]
                assert "ruff" in ruff_cmd
                assert "check" in ruff_cmd

                # Mypy command
                mypy_cmd = calls[2][0][0]
                assert "mypy" in mypy_cmd

    def test_command_with_path(self):
        """n7 dkc pyqa src/"""
        with patch("n7.commands.docker.format.docker_pyqa_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_pyqa_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_pyqa_command, ["src/"])

                assert result.exit_code == 0
                assert mock_run.call_count == 3

                calls = mock_run.call_args_list
                black_cmd = calls[0][0][0]
                ruff_cmd = calls[1][0][0]
                mypy_cmd = calls[2][0][0]

                assert black_cmd[-1] == "src/"
                assert ruff_cmd[-1] == "src/"
                assert mypy_cmd[-1] == "src/"

    def test_command_with_fix(self):
        """n7 dkc pyqa --fix -> black (format), ruff check --fix, mypy"""
        with patch("n7.commands.docker.format.docker_pyqa_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_pyqa_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_pyqa_command, ["--fix"])

                assert result.exit_code == 0
                assert mock_run.call_count == 3

                calls = mock_run.call_args_list
                black_cmd = calls[0][0][0]
                ruff_cmd = calls[1][0][0]
                mypy_cmd = calls[2][0][0]

                # Black sans --check (mode format)
                assert "black" in black_cmd
                assert "--check" not in black_cmd

                # Ruff avec --fix
                assert "ruff" in ruff_cmd
                assert "--fix" in ruff_cmd

                # Mypy reste inchangé
                assert "mypy" in mypy_cmd

    def test_command_with_fix_and_path(self):
        """n7 dkc pyqa --fix src/"""
        with patch("n7.commands.docker.format.docker_pyqa_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_pyqa_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_pyqa_command, ["--fix", "src/"])

                assert result.exit_code == 0
                assert mock_run.call_count == 3

                calls = mock_run.call_args_list
                black_cmd = calls[0][0][0]
                ruff_cmd = calls[1][0][0]
                mypy_cmd = calls[2][0][0]

                assert "src/" in black_cmd
                assert "--check" not in black_cmd
                assert "src/" in ruff_cmd
                assert "--fix" in ruff_cmd
                assert "src/" in mypy_cmd

    def test_command_with_custom_service(self):
        """n7 dkc pyqa --service backend"""
        with patch("n7.commands.docker.format.docker_pyqa_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_pyqa_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_pyqa_command, ["--service", "backend"])

                assert result.exit_code == 0
                assert mock_run.call_count == 3

                calls = mock_run.call_args_list
                black_cmd = calls[0][0][0]
                ruff_cmd = calls[1][0][0]
                mypy_cmd = calls[2][0][0]

                assert "backend" in black_cmd
                assert "backend" in ruff_cmd
                assert "backend" in mypy_cmd

    def test_commands_executed_sequentially(self):
        """Vérifie que les commandes sont exécutées dans l'ordre: black, ruff, mypy"""
        with patch("n7.commands.docker.format.docker_pyqa_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.format.docker_pyqa_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                runner.invoke(docker_pyqa_command, [])

                calls = mock_run.call_args_list
                assert len(calls) == 3

                # Vérifier l'ordre d'exécution via les commandes
                assert "black" in calls[0][0][0]
                assert "ruff" in calls[1][0][0]
                assert "mypy" in calls[2][0][0]



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.docker.test.docker_pytest_command import docker_pytest_command

runner = CliRunner()


class TestDockerTestCommand:
    """Tests pour la commande docker test"""

    def test_command_without_options(self):
        """n7 dkc t -> pytest simple dans container"""
        with patch("n7.commands.docker.test.docker_pytest_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.test.docker_pytest_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_pytest_command, [])

                assert result.exit_code == 0
                mock_run.assert_called_once()
                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "api",
                    "pytest",
                )

    def test_command_with_path(self):
        """n7 dkc t users/tests/"""
        with patch("n7.commands.docker.test.docker_pytest_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.test.docker_pytest_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_pytest_command, ["users/tests/"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert cmd[-1] == "users/tests/"

    def test_command_with_verbose(self):
        """n7 dkc t -v"""
        with patch("n7.commands.docker.test.docker_pytest_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.test.docker_pytest_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_pytest_command, ["-v"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "-v" in cmd

    def test_command_with_workers(self):
        """n7 dkc t -n auto"""
        with patch("n7.commands.docker.test.docker_pytest_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.test.docker_pytest_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_pytest_command, ["-n", "auto"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "-n" in cmd
                assert "auto" in cmd

    def test_command_with_coverage(self):
        """n7 dkc t --cov users"""
        with patch("n7.commands.docker.test.docker_pytest_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.test.docker_pytest_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_pytest_command, ["--cov", "users"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--cov=users" in cmd

    def test_command_with_last_failed(self):
        """n7 dkc t --lf"""
        with patch("n7.commands.docker.test.docker_pytest_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.test.docker_pytest_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_pytest_command, ["--lf"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "--lf" in cmd

    def test_command_with_custom_service(self):
        """n7 dkc t --service worker"""
        with patch("n7.commands.docker.test.docker_pytest_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.test.docker_pytest_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(docker_pytest_command, ["--service", "worker"])

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "worker" in cmd

    def test_command_combined_options(self):
        """n7 dkc t users/tests/ -v -n 4 --cov users"""
        with patch("n7.commands.docker.test.docker_pytest_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.test.docker_pytest_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(
                    docker_pytest_command,
                    [
                        "-v",
                        "-n",
                        "4",
                        "--cov",
                        "users",
                        "users/tests/",  # path à la fin
                    ],
                )

                assert result.exit_code == 0
                cmd = mock_run.call_args[0][0]
                assert "-v" in cmd
                assert "-n" in cmd
                assert "4" in cmd
                assert "--cov=users" in cmd
                assert cmd[-1] == "users/tests/"



import subprocess

import typer

from n7.commands.builders import DockerComposeCommandBuilder
from n7.commands.resolver import DockerFileResolver

logs_command = typer.Typer(help="Gestion containers Docker Compose")


@logs_command.callback(invoke_without_command=True)
def logs(
    service: str = typer.Argument(
        "", help="Name of the service container (optional, all services if not specified)"
    ),
    follow: bool = typer.Option(False, "-f", "--follow", help="Do you want see live file logs"),
):
    """Show logs of Docker Compose containers"""
    # Résolution des fichiers docker
    resolver = DockerFileResolver()
    config = resolver.resolve()

    # Build commande docker compose
    docker_builder = DockerComposeCommandBuilder(
        env_file=config["env_file"] or ".env",
        compose_file=config["compose_file"] or "docker-compose.yml",
    )
    cmd = docker_builder.build_logs(service=service, follow=follow)

    # Exécution
    subprocess.run(cmd, check=True)



"""Commandes du CLI N7"""

from n7.commands.docker.container.connect_postgres_command import connect_postgres_command
from n7.commands.docker.container.connect_shell_command import connect_shell_command
from n7.commands.docker.container.down_command import down_command
from n7.commands.docker.container.logs_command import logs_command
from n7.commands.docker.container.py_manage_command import py_manage_command
from n7.commands.docker.container.up_command import up_command
from n7.commands.docker.format.docker_black_command import docker_black_command
from n7.commands.docker.format.docker_mypy_command import docker_mypy_command
from n7.commands.docker.format.docker_pycheck_command import docker_pycheck_command
from n7.commands.docker.format.docker_pyqa_command import docker_pyqa_command
from n7.commands.docker.format.docker_ruff_command import docker_ruff_command
from n7.commands.docker.format.docker_uv_command import docker_uv_command
from n7.commands.docker.test.docker_pytest_command import docker_pytest_command
from n7.commands.local.format.local_black_command import local_black_command
from n7.commands.local.format.local_mypy_command import local_mypy_command
from n7.commands.local.format.local_pycheck_command import local_pycheck_command
from n7.commands.local.format.local_pyqa_command import local_pyqa_command
from n7.commands.local.format.local_ruff_command import local_ruff_command
from n7.commands.local.format.local_uv_command import local_uv_command
from n7.commands.local.test.local_pytest_command import local_pytest_command

__all__ = [
    "local_pytest_command",
    "local_black_command",
    "local_ruff_command",
    "local_mypy_command",
    "local_pyqa_command",
    "local_pycheck_command",
    "local_uv_command",
    "docker_pytest_command",
    "docker_black_command",
    "docker_ruff_command",
    "docker_mypy_command",
    "docker_pyqa_command",
    "docker_pycheck_command",
    "docker_uv_command",
    "up_command",
    "down_command",
    "logs_command",
    "connect_shell_command",
    "connect_postgres_command",
    "py_manage_command",
]



"""N7 CLI pour les projets N7"""

import typer
from rich.console import Console

from n7 import __version__
from n7.commands import (
    connect_postgres_command,
    connect_shell_command,
    docker_black_command,
    docker_mypy_command,
    docker_pycheck_command,
    docker_pyqa_command,
    docker_pytest_command,
    docker_ruff_command,
    docker_uv_command,
    down_command,
    local_black_command,
    local_mypy_command,
    local_pycheck_command,
    local_pyqa_command,
    local_pytest_command,
    local_ruff_command,
    local_uv_command,
    logs_command,
    py_manage_command,
    up_command,
)

# init Typer
app = typer.Typer(
    name="n7",
    help="CLI global pour les projets N7",
    add_completion=True,
    rich_markup_mode="rich",
    context_settings={"help_option_names": ["-h", "--help"]},
    pretty_exceptions_show_locals=False,
    pretty_exceptions_short=True,
)

console = Console()

# Group docker compose
dkc_app = typer.Typer(help="Commandes Docker Compose")
dkc_app.add_typer(docker_pytest_command, name="pyt", help="Tests pytest in container")
dkc_app.add_typer(
    docker_black_command, name="pybl", help="Format Python code with Black in container"
)
dkc_app.add_typer(docker_ruff_command, name="pyrf", help="Lint Python code with Ruff in container")
dkc_app.add_typer(
    docker_mypy_command, name="pymy", help="Type check Python code with Mypy in container"
)
dkc_app.add_typer(docker_pyqa_command, name="pyqa", help="Run Python QA tools in container")
dkc_app.add_typer(
    docker_pycheck_command, name="pycheck", help="Run Python checks (Tests + QA) in container"
)
dkc_app.add_typer(docker_uv_command, name="uv", help="Run uv command in container")
dkc_app.add_typer(up_command, name="up", help="Start container")
dkc_app.add_typer(down_command, name="down", help="Stop and delete container")
dkc_app.add_typer(logs_command, name="l", help="Logs a container")
dkc_app.add_typer(connect_shell_command, name="sh", help="Start shell in container")
dkc_app.add_typer(connect_postgres_command, name="psql", help="Connect to PostgreSQL database")
dkc_app.add_typer(py_manage_command, name="pymana", help="Run python manage.py in container")

app.add_typer(dkc_app, name="dkc", help="Commandes Docker Compose")

# Group local
app.add_typer(local_pytest_command, name="pyt", help="Tests pytest local")
app.add_typer(local_black_command, name="pybl", help="Format Python code with Black")
app.add_typer(local_ruff_command, name="pyrf", help="Lint Python code with Ruff")
app.add_typer(local_mypy_command, name="pymy", help="Type check Python code with Mypy")
app.add_typer(local_pyqa_command, name="pyqa", help="Run Python QA tools")
app.add_typer(local_pycheck_command, name="pycheck", help="Run Python checks (Tests + QA)")
app.add_typer(local_uv_command, name="uv", help="Run uv command")

#
# HELP => display help
#

version_option = typer.Option(
    False,
    "--version",
    "-v",
    help="Show version CLI n7",
)

debug_option = typer.Option(
    False,
    "--debug",
    help="Mode debug with traceback completed",
)


@app.callback(invoke_without_command=True)
def main_callback(version: bool = version_option, debug: bool = debug_option):
    """Callback principal pour gérer les options globales."""
    if debug:
        app.pretty_exceptions_show_locals = True
        app.pretty_exceptions_short = False

    if version:
        console.print(f"version [blue]{__version__}[/blue]")
        raise typer.Exit()


if __name__ == "__main__":
    app()



class DockerComposeCommandBuilder:
    def __init__(self, env_file: str, compose_file: str):
        self.env_file = env_file
        self.compose_file = compose_file

    def _base(self) -> tuple[str, ...]:
        """Commande de base docker compose"""
        return (
            "docker",
            "compose",
            "--env-file",
            self.env_file,
            "-f",
            self.compose_file,
        )

    def build_exec(
        self,
        service: str,
        cmd: tuple[str, ...],
        disable_tty: bool = False,
    ) -> tuple[str, ...]:
        """Construit docker compose exec"""
        base = self._base()
        if disable_tty:
            return *base, "exec", "-T", service, *cmd
        return *base, "exec", service, *cmd

    def build_up(self, detach: bool = True, build: bool = False) -> tuple[str, ...]:
        """Construit docker compose up"""
        base = self._base()
        if detach and build:
            return *base, "up", "--build", "-d"
        if detach:
            return *base, "up", "-d"
        if build:
            return *base, "up", "--build"
        return *base, "up"

    def build_down(self, volumes: bool = False) -> tuple[str, ...]:
        """Construit docker compose down"""
        base = self._base()
        if volumes:
            return *base, "down", "-v"
        return *base, "down"

    def build_logs(self, service: str, follow: bool = False) -> tuple[str, ...]:
        """Construit docker compose logs"""
        base = self._base()
        if follow:
            return *base, "logs", "-f", service
        return *base, "logs", service



from unittest.mock import patch

from typer.testing import CliRunner

from n7.commands.docker.container.py_manage_command import py_manage_command

runner = CliRunner()


class TestManageCommand:
    """Test pour la commande qui permet d'exécuter python manage.py"""

    def test_manage_command_without_args(self):
        """test manage sans arguments"""
        with patch("n7.commands.docker.container.py_manage_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.py_manage_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(py_manage_command, [])

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "api",
                    "python",
                    "manage.py",
                )

    def test_manage_command_with_single_arg(self):
        """test manage avec un argument"""
        with patch("n7.commands.docker.container.py_manage_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.py_manage_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(py_manage_command, ["migrate"])

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "api",
                    "python",
                    "manage.py",
                    "migrate",
                )

    def test_manage_command_with_multiple_args(self):
        """test manage avec plusieurs arguments"""
        with patch("n7.commands.docker.container.py_manage_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.py_manage_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(py_manage_command, ["makemigrations", "app"])

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "api",
                    "python",
                    "manage.py",
                    "makemigrations",
                    "app",
                )

    def test_manage_command_with_custom_service(self):
        """test manage avec service custom"""
        with patch("n7.commands.docker.container.py_manage_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.py_manage_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(py_manage_command, ["--service", "backend", "migrate"])

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "backend",
                    "python",
                    "manage.py",
                    "migrate",
                )

    def test_manage_command_with_service_and_multiple_args(self):
        """test manage avec service custom et plusieurs arguments"""
        with patch("n7.commands.docker.container.py_manage_command.subprocess.run") as mock_run:
            with patch(
                "n7.commands.docker.container.py_manage_command.DockerFileResolver"
            ) as mock_resolver:
                mock_resolver.return_value.resolve.return_value = {
                    "env_file": ".env",
                    "compose_file": "docker-compose.yml",
                    "default_service": "api",
                }

                result = runner.invoke(
                    py_manage_command,
                    ["--service", "backend", "createsuperuser", "--username", "admin"],
                )

                assert result.exit_code == 0
                mock_run.assert_called_once()

                cmd = mock_run.call_args[0][0]
                assert cmd == (
                    "docker",
                    "compose",
                    "--env-file",
                    ".env",
                    "-f",
                    "docker-compose.yml",
                    "exec",
                    "backend",
                    "python",
                    "manage.py",
                    "createsuperuser",
                    "--username",
                    "admin",
                )
